@@ -15,12 +15,9 @@ use crate::api::InterConfig;
 
 use crossbeam::channel::*;
 
-// use crate::encoder::*;
-use crate::config::CpuFeatureLevel;
-use crate::encoder::Sequence;
 use crate::frame::*;
-use crate::scenechange::SceneChangeDetector;
 use crate::util::Pixel;
+use av_scenechange::SceneChangeDetector;
 
 use std::collections::BTreeMap;
 use std::sync::Arc;
@@ -42,20 +39,38 @@ impl<T: Pixel> SubGop<T> {
 struct SceneChange<T: Pixel> {
   frames: usize,
   pyramid_size: usize,
-  processed: u64,
-  last_keyframe: u64,
+  processed: usize,
+  last_keyframe: usize,
   detector: SceneChangeDetector<T>,
 }
 
 impl<T: Pixel> SceneChange<T> {
   fn new(pyramid_size: usize, enc: &EncoderConfig) -> Self {
-    let seq = Arc::new(Sequence::new(enc));
-
+    let inter_cfg = InterConfig::new(enc);
+    let lookahead_distance = inter_cfg.keyframe_lookahead_distance() as usize;
     let detector = SceneChangeDetector::new(
-      enc.clone(),
-      CpuFeatureLevel::default(),
-      pyramid_size,
-      seq,
+      (enc.width, enc.height),
+      enc.bit_depth,
+      av_scenechange::Rational32::new(
+        enc.time_base.den as i32,
+        enc.time_base.num as i32,
+      ),
+      enc.chroma_sampling,
+      lookahead_distance,
+      match enc.speed_settings.scene_detection_mode {
+        super::SceneDetectionSpeed::Fast => {
+          av_scenechange::SceneDetectionSpeed::Fast
+        }
+        super::SceneDetectionSpeed::Standard => {
+          av_scenechange::SceneDetectionSpeed::Standard
+        }
+        super::SceneDetectionSpeed::None => {
+          av_scenechange::SceneDetectionSpeed::None
+        }
+      },
+      enc.min_key_frame_interval as usize,
+      enc.max_key_frame_interval as usize,
+      av_scenechange::CpuFeatureLevel::default(),
     );
 
     Self { frames: 0, pyramid_size, processed: 0, last_keyframe: 0, detector }
