@@ -1,4 +1,4 @@
-// Copyright (c) 2019-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2019-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -83,6 +83,13 @@ const fn get_2d_mode_idx(mode_x: FilterMode, mode_y: FilterMode) -> usize {
   (mode_x as usize + 4 * (mode_y as usize)) & 15
 }
 
+/// # Panics
+///
+/// - If `width` is not a power of 2
+/// - If `width` is not between 2 and 128
+/// - If `height` is odd
+/// - If `width * height` is greater than the length of `tmp1` or `tmp2`
+/// - If `width` and `height` do not fit within the bounds of `src`
 #[inline(always)]
 pub fn put_8tap<T: Pixel>(
   dst: &mut PlaneRegionMut<'_, T>, src: PlaneSlice<'_, T>, width: usize,
@@ -101,11 +108,24 @@ pub fn put_8tap<T: Pixel>(
     call_rust(&mut copy.as_region_mut());
     copy
   };
-  match T::type_enum() {
-    PixelType::U8 => {
-      match PUT_FNS[cpu.as_index()][get_2d_mode_idx(mode_x, mode_y)] {
-        Some(func) => unsafe {
-          (func)(
+
+  // SAFETY: The assembly only supports even heights and valid uncropped
+  //         widths
+  unsafe {
+    assert_eq!(height & 1, 0);
+    assert!(width.is_power_of_two() && (2..=128).contains(&width));
+
+    // SAFETY: Check bounds of dst
+    assert!(dst.rect().width >= width && dst.rect().height >= height);
+
+    // SAFETY: Check bounds of src
+    assert!(src.accessible(width + 4, height + 4));
+    assert!(src.accessible_neg(3, 3));
+
+    match T::type_enum() {
+      PixelType::U8 => {
+        match PUT_FNS[cpu.as_index()][get_2d_mode_idx(mode_x, mode_y)] {
+          Some(func) => (func)(
             dst.data_ptr_mut() as *mut _,
             T::to_asm_stride(dst.plane_cfg.stride),
             src.as_ptr() as *const _,
@@ -114,15 +134,13 @@ pub fn put_8tap<T: Pixel>(
             height as i32,
             col_frac,
             row_frac,
-          );
-        },
-        None => call_rust(dst),
+          ),
+          None => call_rust(dst),
+        }
       }
-    }
-    PixelType::U16 => {
-      match PUT_HBD_FNS[cpu.as_index()][get_2d_mode_idx(mode_x, mode_y)] {
-        Some(func) => unsafe {
-          (func)(
+      PixelType::U16 => {
+        match PUT_HBD_FNS[cpu.as_index()][get_2d_mode_idx(mode_x, mode_y)] {
+          Some(func) => (func)(
             dst.data_ptr_mut() as *mut _,
             T::to_asm_stride(dst.plane_cfg.stride),
             src.as_ptr() as *const _,
@@ -132,9 +150,9 @@ pub fn put_8tap<T: Pixel>(
             col_frac,
             row_frac,
             (1 << bit_depth) - 1,
-          );
-        },
-        None => call_rust(dst),
+          ),
+          None => call_rust(dst),
+        }
       }
     }
   }
@@ -150,6 +168,13 @@ pub fn put_8tap<T: Pixel>(
   }
 }
 
+/// # Panics
+///
+/// - If `width` is not a power of 2
+/// - If `width` is not between 2 and 128
+/// - If `height` is odd
+/// - If `width * height` is greater than the length of `tmp1` or `tmp2`
+/// - If `width` and `height` do not fit within the bounds of `src`
 #[inline(always)]
 pub fn prep_8tap<T: Pixel>(
   tmp: &mut [i16], src: PlaneSlice<'_, T>, width: usize, height: usize,
@@ -169,11 +194,24 @@ pub fn prep_8tap<T: Pixel>(
     call_rust(&mut copy);
     copy
   };
-  match T::type_enum() {
-    PixelType::U8 => {
-      match PREP_FNS[cpu.as_index()][get_2d_mode_idx(mode_x, mode_y)] {
-        Some(func) => unsafe {
-          (func)(
+
+  // SAFETY: The assembly only supports even heights and valid uncropped
+  //         widths
+  unsafe {
+    assert_eq!(height & 1, 0);
+    assert!(width.is_power_of_two() && (2..=128).contains(&width));
+
+    // SAFETY: Check length of tmp
+    assert!(tmp.len() >= width * height);
+
+    // SAFETY: Check bounds of src
+    assert!(src.accessible(width + 4, height + 4));
+    assert!(src.accessible_neg(3, 3));
+
+    match T::type_enum() {
+      PixelType::U8 => {
+        match PREP_FNS[cpu.as_index()][get_2d_mode_idx(mode_x, mode_y)] {
+          Some(func) => (func)(
             tmp.as_mut_ptr(),
             src.as_ptr() as *const _,
             T::to_asm_stride(src.plane.cfg.stride),
@@ -181,15 +219,13 @@ pub fn prep_8tap<T: Pixel>(
             height as i32,
             col_frac,
             row_frac,
-          );
-        },
-        None => call_rust(tmp),
+          ),
+          None => call_rust(tmp),
+        }
       }
-    }
-    PixelType::U16 if bit_depth > 8 => {
-      match PREP_HBD_FNS[cpu.as_index()][get_2d_mode_idx(mode_x, mode_y)] {
-        Some(func) => unsafe {
-          (func)(
+      PixelType::U16 if bit_depth > 8 => {
+        match PREP_HBD_FNS[cpu.as_index()][get_2d_mode_idx(mode_x, mode_y)] {
+          Some(func) => (func)(
             tmp.as_mut_ptr() as *mut _,
             src.as_ptr() as *const _,
             T::to_asm_stride(src.plane.cfg.stride),
@@ -198,12 +234,12 @@ pub fn prep_8tap<T: Pixel>(
             col_frac,
             row_frac,
             (1 << bit_depth) - 1,
-          );
-        },
-        None => call_rust(tmp),
+          ),
+          None => call_rust(tmp),
+        }
       }
+      _ => call_rust(tmp),
     }
-    _ => call_rust(tmp),
   }
   #[cfg(feature = "check_asm")]
   {
@@ -211,6 +247,12 @@ pub fn prep_8tap<T: Pixel>(
   }
 }
 
+/// # Panics
+///
+/// - If `width` is not a power of 2
+/// - If `width` is not between 2 and 128
+/// - If `width * height` is greater than the length of `tmp1` or `tmp2`
+/// - If `width` and `height` do not fit within the bounds of `dst`
 pub fn mc_avg<T: Pixel>(
   dst: &mut PlaneRegionMut<'_, T>, tmp1: &[i16], tmp2: &[i16], width: usize,
   height: usize, bit_depth: usize, cpu: CpuFeatureLevel,
@@ -224,23 +266,34 @@ pub fn mc_avg<T: Pixel>(
     call_rust(&mut copy.as_region_mut());
     copy
   };
-  match T::type_enum() {
-    PixelType::U8 => match AVG_FNS[cpu.as_index()] {
-      Some(func) => unsafe {
-        (func)(
+
+  // SAFETY: The assembly only supports even heights and valid uncropped
+  //         widths
+  unsafe {
+    assert_eq!(height & 1, 0);
+    assert!(width.is_power_of_two() && (2..=128).contains(&width));
+
+    // SAFETY: Check bounds of dst
+    assert!(dst.rect().width >= width && dst.rect().height >= height);
+
+    // SAFETY: Check length of tmp1 & tmp2
+    assert!(tmp1.len() >= width * height);
+    assert!(tmp2.len() >= width * height);
+
+    match T::type_enum() {
+      PixelType::U8 => match AVG_FNS[cpu.as_index()] {
+        Some(func) => (func)(
           dst.data_ptr_mut() as *mut _,
           T::to_asm_stride(dst.plane_cfg.stride),
           tmp1.as_ptr(),
           tmp2.as_ptr(),
           width as i32,
           height as i32,
-        );
+        ),
+        None => call_rust(dst),
       },
-      None => call_rust(dst),
-    },
-    PixelType::U16 if bit_depth > 8 => match AVG_HBD_FNS[cpu.as_index()] {
-      Some(func) => unsafe {
-        (func)(
+      PixelType::U16 if bit_depth > 8 => match AVG_HBD_FNS[cpu.as_index()] {
+        Some(func) => (func)(
           dst.data_ptr_mut() as *mut _,
           T::to_asm_stride(dst.plane_cfg.stride),
           tmp1.as_ptr(),
@@ -248,11 +301,11 @@ pub fn mc_avg<T: Pixel>(
           width as i32,
           height as i32,
           (1 << bit_depth) - 1,
-        );
+        ),
+        None => call_rust(dst),
       },
-      None => call_rust(dst),
-    },
-    _ => call_rust(dst),
+      _ => call_rust(dst),
+    }
   }
   #[cfg(feature = "check_asm")]
   {
@@ -280,6 +333,11 @@ macro_rules! decl_mc_fns {
             dst: *mut u8, dst_stride: isize, src: *const u8, src_stride: isize,
             w: i32, h: i32, mx: i32, my: i32
           );
+
+          fn [<$func_name _avx512icl>](
+            dst: *mut u8, dst_stride: isize, src: *const u8, src_stride: isize,
+            w: i32, h: i32, mx: i32, my: i32
+          );
         )*
       }
 
@@ -298,27 +356,35 @@ macro_rules! decl_mc_fns {
         )*
         out
       };
+
+      static PUT_FNS_AVX512ICL: [Option<PutFn>; 16] = {
+        let mut out: [Option<PutFn>; 16] = [None; 16];
+        $(
+          out[get_2d_mode_idx($mode_x, $mode_y)] = Some([<$func_name _avx512icl>]);
+        )*
+        out
+      };
     }
   }
 }
 
 decl_mc_fns!(
-  (REGULAR, REGULAR, rav1e_put_8tap_regular),
-  (REGULAR, SMOOTH, rav1e_put_8tap_regular_smooth),
-  (REGULAR, SHARP, rav1e_put_8tap_regular_sharp),
-  (SMOOTH, REGULAR, rav1e_put_8tap_smooth_regular),
-  (SMOOTH, SMOOTH, rav1e_put_8tap_smooth),
-  (SMOOTH, SHARP, rav1e_put_8tap_smooth_sharp),
-  (SHARP, REGULAR, rav1e_put_8tap_sharp_regular),
-  (SHARP, SMOOTH, rav1e_put_8tap_sharp_smooth),
-  (SHARP, SHARP, rav1e_put_8tap_sharp),
-  (BILINEAR, BILINEAR, rav1e_put_bilin)
+  (REGULAR, REGULAR, rav1e_put_8tap_regular_8bpc),
+  (REGULAR, SMOOTH, rav1e_put_8tap_regular_smooth_8bpc),
+  (REGULAR, SHARP, rav1e_put_8tap_regular_sharp_8bpc),
+  (SMOOTH, REGULAR, rav1e_put_8tap_smooth_regular_8bpc),
+  (SMOOTH, SMOOTH, rav1e_put_8tap_smooth_8bpc),
+  (SMOOTH, SHARP, rav1e_put_8tap_smooth_sharp_8bpc),
+  (SHARP, REGULAR, rav1e_put_8tap_sharp_regular_8bpc),
+  (SHARP, SMOOTH, rav1e_put_8tap_sharp_smooth_8bpc),
+  (SHARP, SHARP, rav1e_put_8tap_sharp_8bpc),
+  (BILINEAR, BILINEAR, rav1e_put_bilin_8bpc)
 );
 
 cpu_function_lookup_table!(
   PUT_FNS: [[Option<PutFn>; 16]],
   default: [None; 16],
-  [SSSE3, AVX2]
+  [SSSE3, AVX2, AVX512ICL]
 );
 
 macro_rules! decl_mc_hbd_fns {
@@ -381,6 +447,11 @@ macro_rules! decl_mct_fns {
     paste::item! {
       extern {
         $(
+          fn [<$func_name _sse2>](
+            tmp: *mut i16, src: *const u8, src_stride: libc::ptrdiff_t, w: i32,
+            h: i32, mx: i32, my: i32
+          );
+
           fn [<$func_name _ssse3>](
             tmp: *mut i16, src: *const u8, src_stride: libc::ptrdiff_t, w: i32,
             h: i32, mx: i32, my: i32
@@ -398,6 +469,14 @@ macro_rules! decl_mct_fns {
         )*
       }
 
+      static PREP_FNS_SSE2: [Option<PrepFn>; 16] = {
+        let mut out: [Option<PrepFn>; 16] = [None; 16];
+        $(
+            out[get_2d_mode_idx($mode_x, $mode_y)] = Some([<$func_name _sse2>]);
+        )*
+        out
+      };
+
       static PREP_FNS_SSSE3: [Option<PrepFn>; 16] = {
         let mut out: [Option<PrepFn>; 16] = [None; 16];
         $(
@@ -426,22 +505,22 @@ macro_rules! decl_mct_fns {
 }
 
 decl_mct_fns!(
-  (REGULAR, REGULAR, rav1e_prep_8tap_regular),
-  (REGULAR, SMOOTH, rav1e_prep_8tap_regular_smooth),
-  (REGULAR, SHARP, rav1e_prep_8tap_regular_sharp),
-  (SMOOTH, REGULAR, rav1e_prep_8tap_smooth_regular),
-  (SMOOTH, SMOOTH, rav1e_prep_8tap_smooth),
-  (SMOOTH, SHARP, rav1e_prep_8tap_smooth_sharp),
-  (SHARP, REGULAR, rav1e_prep_8tap_sharp_regular),
-  (SHARP, SMOOTH, rav1e_prep_8tap_sharp_smooth),
-  (SHARP, SHARP, rav1e_prep_8tap_sharp),
-  (BILINEAR, BILINEAR, rav1e_prep_bilin)
+  (REGULAR, REGULAR, rav1e_prep_8tap_regular_8bpc),
+  (REGULAR, SMOOTH, rav1e_prep_8tap_regular_smooth_8bpc),
+  (REGULAR, SHARP, rav1e_prep_8tap_regular_sharp_8bpc),
+  (SMOOTH, REGULAR, rav1e_prep_8tap_smooth_regular_8bpc),
+  (SMOOTH, SMOOTH, rav1e_prep_8tap_smooth_8bpc),
+  (SMOOTH, SHARP, rav1e_prep_8tap_smooth_sharp_8bpc),
+  (SHARP, REGULAR, rav1e_prep_8tap_sharp_regular_8bpc),
+  (SHARP, SMOOTH, rav1e_prep_8tap_sharp_smooth_8bpc),
+  (SHARP, SHARP, rav1e_prep_8tap_sharp_8bpc),
+  (BILINEAR, BILINEAR, rav1e_prep_bilin_8bpc)
 );
 
 cpu_function_lookup_table!(
   PREP_FNS: [[Option<PrepFn>; 16]],
   default: [None; 16],
-  [SSSE3, AVX2, AVX512ICL]
+  [SSE2, SSSE3, AVX2, AVX512ICL]
 );
 
 macro_rules! decl_mct_hbd_fns {
@@ -500,12 +579,17 @@ cpu_function_lookup_table!(
 );
 
 extern {
-  fn rav1e_avg_ssse3(
+  fn rav1e_avg_8bpc_ssse3(
     dst: *mut u8, dst_stride: libc::ptrdiff_t, tmp1: *const i16,
     tmp2: *const i16, w: i32, h: i32,
   );
 
-  fn rav1e_avg_avx2(
+  fn rav1e_avg_8bpc_avx2(
+    dst: *mut u8, dst_stride: libc::ptrdiff_t, tmp1: *const i16,
+    tmp2: *const i16, w: i32, h: i32,
+  );
+
+  fn rav1e_avg_8bpc_avx512icl(
     dst: *mut u8, dst_stride: libc::ptrdiff_t, tmp1: *const i16,
     tmp2: *const i16, w: i32, h: i32,
   );
@@ -524,7 +608,11 @@ extern {
 cpu_function_lookup_table!(
   AVG_FNS: [Option<AvgFn>],
   default: None,
-  [(SSSE3, Some(rav1e_avg_ssse3)), (AVX2, Some(rav1e_avg_avx2))]
+  [
+    (SSSE3, Some(rav1e_avg_8bpc_ssse3)),
+    (AVX2, Some(rav1e_avg_8bpc_avx2)),
+    (AVX512ICL, Some(rav1e_avg_8bpc_avx512icl))
+  ]
 );
 
 cpu_function_lookup_table!(
@@ -545,7 +633,7 @@ mod test {
         paste::item! {
           #[test]
           fn [<test_ $func_name _bd_ $BD _ $OPT>]() {
-            if !is_x86_feature_detected!($OPTLIT) {
+            if CpuFeatureLevel::default() < CpuFeatureLevel::from_str($OPTLIT).unwrap() {
               eprintln!("Ignoring {} test, not supported on this machine!", $OPTLIT);
               return;
             }
@@ -632,7 +720,7 @@ mod test {
         paste::item! {
           #[test]
           fn [<test_ $func_name _bd_ $BD _ $OPT>]() {
-            if !is_x86_feature_detected!($OPTLIT) {
+            if CpuFeatureLevel::default() < CpuFeatureLevel::from_str($OPTLIT).unwrap() {
               eprintln!("Ignoring {} test, not supported on this machine!", $OPTLIT);
               return;
             }
