@@ -158,13 +158,12 @@ unsafe fn dequantize_avx2(
 #[cfg(test)]
 mod test {
   use super::*;
-  use rand::distributions::{Distribution, Uniform};
-  use rand::{thread_rng, Rng};
-  use std::mem::MaybeUninit;
+  use rand::distr::{Distribution, Uniform};
+  use rand::{rng, Rng};
 
   #[test]
   fn dequantize_test() {
-    let mut rng = thread_rng();
+    let mut rng = rng();
 
     use TxSize::*;
     let tx_sizes = [
@@ -176,7 +175,7 @@ mod test {
     let bd: usize = 8;
 
     for &tx_size in &tx_sizes {
-      let qindex: u8 = rng.gen_range((MINQ as u8)..(MAXQ as u8));
+      let qindex: u8 = rng.random_range((MINQ as u8)..(MAXQ as u8));
       let dc_quant = dc_q(qindex, 0, bd).get() as i16;
       let ac_quant = ac_q(qindex, 0, bd).get() as i16;
 
@@ -187,7 +186,7 @@ mod test {
         out[0] = 0;
         out[1] = area as u16;
         for eob in out.iter_mut().skip(2) {
-          *eob = rng.gen_range(0..area as u16);
+          *eob = rng.random_range(0..area as u16);
         }
         out
       };
@@ -197,7 +196,8 @@ mod test {
         let mut rcoeffs = Aligned::new([MaybeUninit::new(0i16); 32 * 32]);
 
         // Generate quantized coefficients up to the eob
-        let between = Uniform::from(-i16::MAX..=i16::MAX);
+        // SAFETY: always correct
+        let between = Uniform::new_inclusive(-i16::MAX, i16::MAX).unwrap();
         for (i, qcoeff) in
           qcoeffs.data.iter_mut().enumerate().take(eob as usize)
         {
