@@ -7,80 +7,6 @@
 // Media Patent License 1.0 was not distributed with this source code in the
 // PATENTS file, you can obtain it at www.aomedia.org/license/patent.
 
-// Safety lints
-#![deny(bare_trait_objects)]
-#![deny(clippy::as_ptr_cast_mut)]
-#![deny(clippy::large_stack_arrays)]
-// Performance lints
-#![warn(clippy::inefficient_to_string)]
-#![warn(clippy::invalid_upcast_comparisons)]
-#![warn(clippy::iter_with_drain)]
-#![warn(clippy::linkedlist)]
-#![warn(clippy::mutex_integer)]
-#![warn(clippy::naive_bytecount)]
-#![warn(clippy::needless_bitwise_bool)]
-#![warn(clippy::needless_collect)]
-#![warn(clippy::or_fun_call)]
-#![warn(clippy::stable_sort_primitive)]
-#![warn(clippy::suboptimal_flops)]
-#![warn(clippy::trivial_regex)]
-#![warn(clippy::trivially_copy_pass_by_ref)]
-#![warn(clippy::unnecessary_join)]
-#![warn(clippy::unused_async)]
-#![warn(clippy::zero_sized_map_values)]
-// Correctness lints
-#![deny(clippy::case_sensitive_file_extension_comparisons)]
-#![deny(clippy::copy_iterator)]
-#![deny(clippy::expl_impl_clone_on_copy)]
-#![deny(clippy::float_cmp)]
-#![warn(clippy::imprecise_flops)]
-#![deny(clippy::manual_instant_elapsed)]
-#![deny(clippy::mem_forget)]
-#![deny(clippy::path_buf_push_overwrite)]
-#![deny(clippy::same_functions_in_if_condition)]
-#![deny(clippy::unchecked_duration_subtraction)]
-#![deny(clippy::unicode_not_nfc)]
-// Clarity/formatting lints
-#![warn(clippy::checked_conversions)]
-#![allow(clippy::comparison_chain)]
-#![warn(clippy::derive_partial_eq_without_eq)]
-#![allow(clippy::enum_variant_names)]
-#![warn(clippy::explicit_deref_methods)]
-#![warn(clippy::filter_map_next)]
-#![warn(clippy::flat_map_option)]
-#![warn(clippy::fn_params_excessive_bools)]
-#![warn(clippy::implicit_clone)]
-#![warn(clippy::iter_not_returning_iterator)]
-#![warn(clippy::iter_on_empty_collections)]
-#![warn(clippy::macro_use_imports)]
-#![warn(clippy::manual_clamp)]
-#![warn(clippy::manual_let_else)]
-#![warn(clippy::manual_ok_or)]
-#![warn(clippy::manual_string_new)]
-#![warn(clippy::map_flatten)]
-#![warn(clippy::match_bool)]
-#![warn(clippy::mut_mut)]
-#![warn(clippy::needless_borrow)]
-#![warn(clippy::needless_continue)]
-#![allow(clippy::needless_range_loop)]
-#![allow(clippy::too_many_arguments)]
-#![warn(clippy::range_minus_one)]
-#![warn(clippy::range_plus_one)]
-#![warn(clippy::ref_binding_to_reference)]
-#![warn(clippy::ref_option_ref)]
-#![warn(clippy::trait_duplication_in_bounds)]
-#![warn(clippy::unused_peekable)]
-#![warn(clippy::unused_rounding)]
-#![warn(clippy::unused_self)]
-#![allow(clippy::upper_case_acronyms)]
-#![warn(clippy::verbose_bit_mask)]
-#![warn(clippy::verbose_file_reads)]
-// Documentation lints
-#![warn(clippy::doc_link_with_quotes)]
-#![warn(clippy::doc_markdown)]
-#![warn(clippy::missing_errors_doc)]
-#![warn(clippy::missing_panics_doc)]
-
 #[macro_use]
 extern crate log;
 
@@ -365,7 +291,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
 
   #[cfg(feature = "tracing")]
   {
-    use tracing_subscriber::layer::subscriberext;
+    use tracing_subscriber::layer::SubscriberExt;
     tracing::subscriber::set_global_default(
       tracing_subscriber::registry().with(chrome_layer),
     )
