@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -11,6 +11,7 @@ use super::*;
 use crate::predict::PredictionMode;
 use crate::predict::PredictionMode::*;
 use crate::transform::TxType::*;
+use std::mem::MaybeUninit;
 
 pub const MAX_TX_SIZE: usize = 64;
 
@@ -240,14 +241,14 @@ pub const TX_PAD_2D: usize = (MAX_CODED_TX_SIZE + TX_PAD_HOR)
 
 const TX_CLASSES: usize = 3;
 
-#[derive(Copy, Clone, PartialEq)]
+#[derive(Copy, Clone, PartialEq, Eq)]
 pub enum TxClass {
   TX_CLASS_2D = 0,
   TX_CLASS_HORIZ = 1,
   TX_CLASS_VERT = 2,
 }
 
-#[derive(Copy, Clone, PartialEq)]
+#[derive(Copy, Clone, PartialEq, Eq)]
 pub enum SegLvl {
   SEG_LVL_ALT_Q = 0,      /* Use alternate Quantizer .... */
   SEG_LVL_ALT_LF_Y_V = 1, /* Use alternate loop filter value on y plane vertical */
@@ -522,7 +523,10 @@ pub struct TXB_CTX {
   pub dc_sign_ctx: usize,
 }
 
-impl<'a> ContextWriter<'a> {
+impl ContextWriter<'_> {
+  /// # Panics
+  ///
+  /// - If an invalid combination of `tx_type` and `tx_size` is passed
   pub fn write_tx_type<W: Writer>(
     &mut self, w: &mut W, tx_size: TxSize, tx_type: TxType,
     y_mode: PredictionMode, is_inter: bool, use_reduced_tx_set: bool,
@@ -540,14 +544,14 @@ impl<'a> ContextWriter<'a> {
       if is_inter {
         let s = av1_tx_ind[tx_set as usize][tx_type as usize] as u32;
         if tx_set_index == 1 {
-          let cdf = &mut self.fc.inter_tx_1_cdf[square_tx_size as usize];
+          let cdf = &self.fc.inter_tx_1_cdf[square_tx_size as usize];
           symbol_with_update!(self, w, s, cdf);
         } else if tx_set_index == 2 {
-          let cdf = &mut self.fc.inter_tx_2_cdf[square_tx_size as usize];
+          let cdf = &self.fc.inter_tx_2_cdf[square_tx_size as usize];
           symbol_with_update!(self, w, s, cdf);
         } else {
-          let cdf = &mut self.fc.inter_tx_3_cdf[square_tx_size as usize];
-          symbol_with_update!(self, w, s, cdf, 2);
+          let cdf = &self.fc.inter_tx_3_cdf[square_tx_size as usize];
+          symbol_with_update!(self, w, s, cdf);
         }
       } else {
         let intra_dir = y_mode;
@@ -557,11 +561,11 @@ impl<'a> ContextWriter<'a> {
 
         let s = av1_tx_ind[tx_set as usize][tx_type as usize] as u32;
         if tx_set_index == 1 {
-          let cdf = &mut self.fc.intra_tx_1_cdf[square_tx_size as usize]
+          let cdf = &self.fc.intra_tx_1_cdf[square_tx_size as usize]
             [intra_dir as usize];
           symbol_with_update!(self, w, s, cdf);
         } else {
-          let cdf = &mut self.fc.intra_tx_2_cdf[square_tx_size as usize]
+          let cdf = &self.fc.intra_tx_2_cdf[square_tx_size as usize]
             [intra_dir as usize];
           symbol_with_update!(self, w, s, cdf);
         }
@@ -577,8 +581,8 @@ impl<'a> ContextWriter<'a> {
     let max_tx_high = max_tx_size.height() as u8;
     let has_above = bo.0.y > 0;
     let has_left = bo.0.x > 0;
-    let mut above = self.bc.above_tx_context[bo.0.x] >= max_tx_wide as u8;
-    let mut left = self.bc.left_tx_context[bo.y_in_sb()] >= max_tx_high as u8;
+    let mut above = self.bc.above_tx_context[bo.0.x] >= max_tx_wide;
+    let mut left = self.bc.left_tx_context[bo.y_in_sb()] >= max_tx_high;
 
     if has_above {
       let above_blk = self.bc.blocks.above_of(bo);
@@ -654,11 +658,11 @@ impl<'a> ContextWriter<'a> {
     debug_assert!(!tx_size.is_rect() || bsize.is_rect_tx_allowed());
 
     if tx_size_cat > 0 {
-      let cdf = &mut self.fc.tx_size_cdf[tx_size_cat - 1][tx_size_ctx];
-      symbol_with_update!(self, w, depth as u32, cdf, 3);
+      let cdf = &self.fc.tx_size_cdf[tx_size_cat - 1][tx_size_ctx];
+      symbol_with_update!(self, w, depth as u32, cdf);
     } else {
-      let cdf = &mut self.fc.tx_size_8x8_cdf[tx_size_ctx];
-      symbol_with_update!(self, w, depth as u32, cdf, 2);
+      let cdf = &self.fc.tx_size_8x8_cdf[tx_size_ctx];
+      symbol_with_update!(self, w, depth as u32, cdf);
     }
   }
 
@@ -713,7 +717,7 @@ impl<'a> ContextWriter<'a> {
 
     let max_tx_size: TxSize = bsize.tx_size().sqr_up();
     let category: usize = (tx_size.sqr_up() != max_tx_size) as usize
-      + (TxSize::TX_SIZES as usize - 1 - max_tx_size as usize) * 2;
+      + (TxSize::TX_SIZES - 1 - max_tx_size as usize) * 2;
 
     debug_assert!(category < TXFM_PARTITION_CONTEXTS);
 
@@ -733,8 +737,8 @@ impl<'a> ContextWriter<'a> {
 
     if tx_size != TX_4X4 && depth < MAX_VARTX_DEPTH {
       let ctx = self.txfm_partition_context(bo, bsize, tx_size, tbx, tby);
-      let cdf = &mut self.fc.txfm_partition_cdf[ctx];
-      symbol_with_update!(self, w, txfm_split as u32, cdf, 2);
+      let cdf = &self.fc.txfm_partition_cdf[ctx];
+      symbol_with_update!(self, w, txfm_split as u32, cdf);
     } else {
       debug_assert!(!txfm_split);
     }
@@ -769,7 +773,7 @@ impl<'a> ContextWriter<'a> {
   }
 
   #[inline]
-  pub fn get_txsize_entropy_ctx(tx_size: TxSize) -> usize {
+  pub const fn get_txsize_entropy_ctx(tx_size: TxSize) -> usize {
     (tx_size.sqr() as usize + tx_size.sqr_up() as usize + 1) >> 1
   }
 
@@ -779,10 +783,10 @@ impl<'a> ContextWriter<'a> {
   ) {
     // Coefficients and levels are transposed from how they work in the spec
     for (coeffs_col, levels_col) in
-      coeffs.chunks(height).zip(levels.chunks_mut(levels_stride))
+      coeffs.chunks_exact(height).zip(levels.chunks_exact_mut(levels_stride))
     {
-      for (coeff, level) in coeffs_col.iter().zip(levels_col.iter_mut()) {
-        *level = clamp(coeff.abs(), T::cast_from(0), T::cast_from(127)).as_();
+      for (coeff, level) in coeffs_col.iter().zip(levels_col) {
+        *level = coeff.abs().min(T::cast_from(127)).as_();
       }
     }
   }
@@ -791,22 +795,27 @@ impl<'a> ContextWriter<'a> {
   // work in the spec, use the log of block height in our calculations instead
   // of block width.
   #[inline]
-  pub fn get_txb_bhl(tx_size: TxSize) -> usize {
+  pub const fn get_txb_bhl(tx_size: TxSize) -> usize {
     av1_get_coded_tx_size(tx_size).height_log2()
   }
 
+  /// Returns `(eob_pt, eob_extra)`
+  ///
+  /// # Panics
+  ///
+  /// - If `eob` is prior to the start of the group
   #[inline]
-  pub fn get_eob_pos_token(eob: usize, extra: &mut u32) -> u32 {
+  pub fn get_eob_pos_token(eob: u16) -> (u32, u32) {
     let t = if eob < 33 {
-      eob_to_pos_small[eob] as u32
+      eob_to_pos_small[usize::from(eob)] as u32
     } else {
-      let e = cmp::min((eob - 1) >> 5, 16);
-      eob_to_pos_large[e as usize] as u32
+      let e = usize::from(cmp::min((eob - 1) >> 5, 16));
+      eob_to_pos_large[e] as u32
     };
     assert!(eob as i32 >= k_eob_group_start[t as usize] as i32);
-    *extra = eob as u32 - k_eob_group_start[t as usize] as u32;
+    let extra = eob as u32 - k_eob_group_start[t as usize] as u32;
 
-    t
+    (t, extra)
   }
 
   pub fn get_nz_mag(levels: &[u8], bhl: usize, tx_class: TxClass) -> usize {
@@ -897,25 +906,33 @@ impl<'a> ContextWriter<'a> {
     Self::get_nz_map_ctx_from_stats(stats, coeff_idx, bhl, tx_size, tx_class)
   }
 
-  pub fn get_nz_map_contexts(
+  /// `coeff_contexts_no_scan` is not in the scan order.
+  /// Value for `pos = scan[i]` is at `coeff[i]`, not at `coeff[pos]`.
+  pub fn get_nz_map_contexts<'c>(
     &self, levels: &mut [u8], scan: &[u16], eob: u16, tx_size: TxSize,
-    tx_class: TxClass, coeff_contexts: &mut [i8],
-  ) {
+    tx_class: TxClass, coeff_contexts_no_scan: &'c mut [MaybeUninit<i8>],
+  ) -> &'c mut [i8] {
     let bhl = Self::get_txb_bhl(tx_size);
     let area = av1_get_coded_tx_size(tx_size).area();
-    for i in 0..eob {
-      let pos = scan[i as usize];
-      coeff_contexts[pos as usize] = Self::get_nz_map_ctx(
+
+    let scan = &scan[..usize::from(eob)];
+    let coeffs = &mut coeff_contexts_no_scan[..usize::from(eob)];
+    for (i, (coeff, pos)) in
+      coeffs.iter_mut().zip(scan.iter().copied()).enumerate()
+    {
+      coeff.write(Self::get_nz_map_ctx(
         levels,
         pos as usize,
         bhl,
         area,
-        i as usize,
-        i == eob - 1,
+        i,
+        i == usize::from(eob) - 1,
         tx_size,
         tx_class,
-      ) as i8;
+      ) as i8);
     }
+    // SAFETY: every element has been initialized
+    unsafe { slice_assume_init_mut(coeffs) }
   }
 
   pub fn get_br_ctx(
