@@ -1106,7 +1106,7 @@ fn filter_v_edge<T: Pixel>(
   } else {
     block.bsize.largest_chroma_tx_size(xdec, ydec)
   };
-  let tx_edge = bo.0.x >> xdec & (txsize.width_mi() - 1) == 0;
+  let tx_edge = (bo.0.x >> xdec) & (txsize.width_mi() - 1) == 0;
   if tx_edge {
     let prev_block = deblock_left(blocks, bo, &p.as_const());
     let block_edge = bo.0.x & (block.n4_w as usize - 1) == 0;
@@ -1153,7 +1153,7 @@ fn sse_v_edge<T: Pixel>(
   } else {
     block.bsize.largest_chroma_tx_size(xdec, ydec)
   };
-  let tx_edge = bo.0.x >> xdec & (txsize.width_mi() - 1) == 0;
+  let tx_edge = (bo.0.x >> xdec) & (txsize.width_mi() - 1) == 0;
   if tx_edge {
     let prev_block = deblock_left(blocks, bo, rec_plane);
     let block_edge = bo.0.x & (block.n4_w as usize - 1) == 0;
@@ -1202,7 +1202,7 @@ fn filter_h_edge<T: Pixel>(
   } else {
     block.bsize.largest_chroma_tx_size(xdec, ydec)
   };
-  let tx_edge = bo.0.y >> ydec & (txsize.height_mi() - 1) == 0;
+  let tx_edge = (bo.0.y >> ydec) & (txsize.height_mi() - 1) == 0;
   if tx_edge {
     let prev_block = deblock_up(blocks, bo, &p.as_const());
     let block_edge = bo.0.y & (block.n4_h as usize - 1) == 0;
@@ -1249,7 +1249,7 @@ fn sse_h_edge<T: Pixel>(
   } else {
     block.bsize.largest_chroma_tx_size(xdec, ydec)
   };
-  let tx_edge = bo.0.y >> ydec & (txsize.height_mi() - 1) == 0;
+  let tx_edge = (bo.0.y >> ydec) & (txsize.height_mi() - 1) == 0;
   if tx_edge {
     let prev_block = deblock_up(blocks, bo, rec_plane);
     let block_edge = bo.0.y & (block.n4_h as usize - 1) == 0;
