@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -11,6 +11,8 @@
 #![allow(non_camel_case_types)]
 #![allow(dead_code)]
 
+use std::mem::MaybeUninit;
+
 cfg_if::cfg_if! {
   if #[cfg(nasm_x86_64)] {
     pub use crate::asm::x86::predict::*;
@@ -21,7 +23,9 @@ cfg_if::cfg_if! {
   }
 }
 
-use crate::context::{MAX_SB_SIZE_LOG2, MAX_TX_SIZE};
+use aligned_vec::{avec, ABox};
+
+use crate::context::{TileBlockOffset, MAX_SB_SIZE_LOG2, MAX_TX_SIZE};
 use crate::cpu_features::CpuFeatureLevel;
 use crate::encoder::FrameInvariants;
 use crate::frame::*;
@@ -30,7 +34,6 @@ use crate::partition::*;
 use crate::tiling::*;
 use crate::transform::*;
 use crate::util::*;
-use std::convert::TryInto;
 
 pub const ANGLE_STEP: i8 = 3;
 
@@ -68,9 +71,10 @@ pub static RAV1E_INTER_COMPOUND_MODES: &[PredictionMode] = &[
 
 // There are more modes than in the spec because every allowed
 // drl index for NEAR modes is considered its own mode.
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd, Eq, Ord)]
+#[derive(Copy, Clone, Debug, PartialEq, PartialOrd, Eq, Ord, Default)]
 pub enum PredictionMode {
-  DC_PRED,     // Average of above and left pixels
+  #[default]
+  DC_PRED, // Average of above and left pixels
   V_PRED,      // Vertical
   H_PRED,      // Horizontal
   D45_PRED,    // Directional 45  degree
@@ -121,7 +125,7 @@ pub enum PredictionVariant {
 
 impl PredictionVariant {
   #[inline]
-  fn new(x: usize, y: usize) -> Self {
+  const fn new(x: usize, y: usize) -> Self {
     match (x, y) {
       (0, 0) => PredictionVariant::NONE,
       (_, 0) => PredictionVariant::LEFT,
@@ -131,14 +135,7 @@ impl PredictionVariant {
   }
 }
 
-impl Default for PredictionMode {
-  #[inline]
-  fn default() -> Self {
-    PredictionMode::DC_PRED
-  }
-}
-
-pub fn intra_mode_to_angle(mode: PredictionMode) -> isize {
+pub const fn intra_mode_to_angle(mode: PredictionMode) -> isize {
   match mode {
     PredictionMode::V_PRED => 90,
     PredictionMode::H_PRED => 180,
@@ -201,11 +198,15 @@ impl PredictionMode {
       1
     }
   }
+
+  /// # Panics
+  ///
+  /// - If called on an inter `PredictionMode`
   pub fn predict_intra<T: Pixel>(
     self, tile_rect: TileRect, dst: &mut PlaneRegionMut<'_, T>,
     tx_size: TxSize, bit_depth: usize, ac: &[i16], intra_param: IntraParam,
-    ief_params: Option<IntraEdgeFilterParameters>,
-    edge_buf: &Aligned<[T; 4 * MAX_TX_SIZE + 1]>, cpu: CpuFeatureLevel,
+    ief_params: Option<IntraEdgeFilterParameters>, edge_buf: &IntraEdge<T>,
+    cpu: CpuFeatureLevel,
   ) {
     assert!(self.is_intra());
     let &Rect { x: frame_x, y: frame_y, .. } = dst.rect();
@@ -263,7 +264,7 @@ impl PredictionMode {
   }
 
   #[inline(always)]
-  pub fn angle_delta_count(self) -> i8 {
+  pub const fn angle_delta_count(self) -> i8 {
     match self {
       PredictionMode::V_PRED
       | PredictionMode::H_PRED
@@ -296,6 +297,10 @@ impl PredictionMode {
   }
 
   /// Inter prediction with a single reference (i.e. not compound mode)
+  ///
+  /// # Panics
+  ///
+  /// - If called on an intra `PredictionMode`
   pub fn predict_inter_single<T: Pixel>(
     self, fi: &FrameInvariants<T>, tile_rect: TileRect, p: usize,
     po: PlaneOffset, dst: &mut PlaneRegionMut<'_, T>, width: usize,
@@ -327,6 +332,10 @@ impl PredictionMode {
   }
 
   /// Inter prediction with two references.
+  ///
+  /// # Panics
+  ///
+  /// - If called on an intra `PredictionMode`
   pub fn predict_inter_compound<T: Pixel>(
     self, fi: &FrameInvariants<T>, tile_rect: TileRect, p: usize,
     po: PlaneOffset, dst: &mut PlaneRegionMut<'_, T>, width: usize,
@@ -373,7 +382,7 @@ impl PredictionMode {
   }
 
   /// Inter prediction that determines whether compound mode is being used based
-  /// on the second ['RefType'] in ['ref_frames'].
+  /// on the second [`RefType`] in [`ref_frames`].
   pub fn predict_inter<T: Pixel>(
     self, fi: &FrameInvariants<T>, tile_rect: TileRect, p: usize,
     po: PlaneOffset, dst: &mut PlaneRegionMut<'_, T>, width: usize,
@@ -413,10 +422,10 @@ impl PredictionMode {
 }
 
 /// A pair of buffers holding the interpolation of two references. Use for
-/// compound inter_prediction
+/// compound inter prediction.
 #[derive(Debug)]
 pub struct InterCompoundBuffers {
-  data: AlignedBoxedSlice<i16>,
+  data: ABox<[i16]>,
 }
 
 impl InterCompoundBuffers {
@@ -445,11 +454,11 @@ impl InterCompoundBuffers {
 
 impl Default for InterCompoundBuffers {
   fn default() -> Self {
-    Self { data: AlignedBoxedSlice::new(2 * Self::BUFFER_SIZE, 0) }
+    Self { data: avec![0; 2 * Self::BUFFER_SIZE].into_boxed_slice() }
   }
 }
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum InterIntraMode {
   II_DC_PRED,
   II_V_PRED,
@@ -458,7 +467,7 @@ pub enum InterIntraMode {
   INTERINTRA_MODES,
 }
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum CompoundType {
   COMPOUND_AVERAGE,
   COMPOUND_WEDGE,
@@ -466,7 +475,7 @@ pub enum CompoundType {
   COMPOUND_TYPES,
 }
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum MotionMode {
   SIMPLE_TRANSLATION,
   OBMC_CAUSAL,   // 2-sided OBMC
@@ -474,7 +483,7 @@ pub enum MotionMode {
   MOTION_MODES,
 }
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum PaletteSize {
   TWO_COLORS,
   THREE_COLORS,
@@ -486,7 +495,7 @@ pub enum PaletteSize {
   PALETTE_SIZES,
 }
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum PaletteColor {
   PALETTE_COLOR_ONE,
   PALETTE_COLOR_TWO,
@@ -499,7 +508,7 @@ pub enum PaletteColor {
   PALETTE_COLORS,
 }
 
-#[derive(Copy, Clone, Debug, PartialEq, PartialOrd)]
+#[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd)]
 pub enum FilterIntraMode {
   FILTER_DC_PRED,
   FILTER_V_PRED,
@@ -522,7 +531,7 @@ pub struct AngleDelta {
   pub uv: i8,
 }
 
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, Default)]
 pub struct IntraEdgeFilterParameters {
   pub plane: usize,
   pub above_ref_frame_types: Option<[RefType; 2]>,
@@ -559,6 +568,9 @@ impl IntraEdgeFilterParameters {
     }
   }
 
+  /// # Panics
+  ///
+  /// - If the appropriate ref frame types are not set on `self`
   pub fn use_smooth_filter(self) -> bool {
     let above_smooth = match self.above_mode {
       Some(PredictionMode::SMOOTH_PRED)
@@ -611,7 +623,7 @@ static sm_weight_arrays: [u8; 2 * MAX_TX_SIZE] = [
 ];
 
 #[inline(always)]
-fn get_scaled_luma_q0(alpha_q3: i16, ac_pred_q3: i16) -> i32 {
+const fn get_scaled_luma_q0(alpha_q3: i16, ac_pred_q3: i16) -> i32 {
   let scaled_luma_q6 = (alpha_q3 as i32) * (ac_pred_q3 as i32);
   let abs_scaled_luma_q0 = (scaled_luma_q6.abs() + 32) >> 6;
   if scaled_luma_q6 < 0 {
@@ -621,14 +633,72 @@ fn get_scaled_luma_q0(alpha_q3: i16, ac_pred_q3: i16) -> i32 {
   }
 }
 
+/// # Returns
+///
+/// Initialized luma AC coefficients
+///
+/// # Panics
+///
+/// - If the block size is invalid for subsampling
+///
+pub fn luma_ac<'ac, T: Pixel>(
+  ac: &'ac mut [MaybeUninit<i16>], ts: &mut TileStateMut<'_, T>,
+  tile_bo: TileBlockOffset, bsize: BlockSize, tx_size: TxSize,
+  fi: &FrameInvariants<T>,
+) -> &'ac mut [i16] {
+  use crate::context::MI_SIZE_LOG2;
+
+  let PlaneConfig { xdec, ydec, .. } = ts.input.planes[1].cfg;
+  let plane_bsize = bsize.subsampled_size(xdec, ydec).unwrap();
+
+  // ensure ac has the right length, so there aren't any uninitialized elements at the end
+  let ac = &mut ac[..plane_bsize.area()];
+
+  let bo = if bsize.is_sub8x8(xdec, ydec) {
+    let offset = bsize.sub8x8_offset(xdec, ydec);
+    tile_bo.with_offset(offset.0, offset.1)
+  } else {
+    tile_bo
+  };
+  let rec = &ts.rec.planes[0];
+  let luma = &rec.subregion(Area::BlockStartingAt { bo: bo.0 });
+  let frame_bo = ts.to_frame_block_offset(bo);
+
+  let frame_clipped_bw: usize =
+    ((fi.w_in_b - frame_bo.0.x) << MI_SIZE_LOG2).min(bsize.width());
+  let frame_clipped_bh: usize =
+    ((fi.h_in_b - frame_bo.0.y) << MI_SIZE_LOG2).min(bsize.height());
+
+  // Similar to 'MaxLumaW' and 'MaxLumaH' stated in https://aomediacodec.github.io/av1-spec/#transform-block-semantics
+  let max_luma_w = if bsize.width() > BlockSize::BLOCK_8X8.width() {
+    let txw_log2 = tx_size.width_log2();
+    ((frame_clipped_bw + (1 << txw_log2) - 1) >> txw_log2) << txw_log2
+  } else {
+    bsize.width()
+  };
+  let max_luma_h = if bsize.height() > BlockSize::BLOCK_8X8.height() {
+    let txh_log2 = tx_size.height_log2();
+    ((frame_clipped_bh + (1 << txh_log2) - 1) >> txh_log2) << txh_log2
+  } else {
+    bsize.height()
+  };
+
+  let w_pad = (bsize.width() - max_luma_w) >> (2 + xdec);
+  let h_pad = (bsize.height() - max_luma_h) >> (2 + ydec);
+  let cpu = fi.cpu_feature_level;
+
+  (match (xdec, ydec) {
+    (0, 0) => pred_cfl_ac::<T, 0, 0>,
+    (1, 0) => pred_cfl_ac::<T, 1, 0>,
+    (_, _) => pred_cfl_ac::<T, 1, 1>,
+  })(ac, luma, plane_bsize, w_pad, h_pad, cpu);
+
+  // SAFETY: it relies on individual pred_cfl_ac implementations to initialize the ac
+  unsafe { slice_assume_init_mut(ac) }
+}
+
 pub(crate) mod rust {
   use super::*;
-  use crate::context::MAX_TX_SIZE;
-  use crate::cpu_features::CpuFeatureLevel;
-  use crate::tiling::PlaneRegionMut;
-  use crate::transform::TxSize;
-  use crate::util::{round_shift, Aligned};
-  use crate::Pixel;
   use std::mem::size_of;
 
   #[inline(always)]
@@ -636,18 +706,18 @@ pub(crate) mod rust {
     mode: PredictionMode, variant: PredictionVariant,
     dst: &mut PlaneRegionMut<'_, T>, tx_size: TxSize, bit_depth: usize,
     ac: &[i16], angle: isize, ief_params: Option<IntraEdgeFilterParameters>,
-    edge_buf: &Aligned<[T; 4 * MAX_TX_SIZE + 1]>, _cpu: CpuFeatureLevel,
+    edge_buf: &IntraEdge<T>, _cpu: CpuFeatureLevel,
   ) {
     let width = tx_size.width();
     let height = tx_size.height();
 
     // left pixels are ordered from bottom to top and right-aligned
-    let (left, not_left) = edge_buf.data.split_at(2 * MAX_TX_SIZE);
-    let (top_left, above) = not_left.split_at(1);
+    let (left, top_left, above) = edge_buf.as_slices();
 
-    let above_slice = &above[..width + height];
-    let left_slice = &left[2 * MAX_TX_SIZE - height..];
-    let left_and_left_below_slice = &left[2 * MAX_TX_SIZE - height - width..];
+    let above_slice = above;
+    let left_slice = &left[left.len().saturating_sub(height)..];
+    let left_and_left_below_slice =
+      &left[left.len().saturating_sub(width + height)..];
 
     match mode {
       PredictionMode::DC_PRED => {
@@ -727,9 +797,7 @@ pub(crate) mod rust {
     let avg = T::cast_from(avg);
 
     for line in output.rows_iter_mut().take(height) {
-      for v in &mut line[..width] {
-        *v = avg;
-      }
+      line[..width].fill(avg);
     }
   }
 
@@ -738,10 +806,8 @@ pub(crate) mod rust {
     width: usize, height: usize, bit_depth: usize,
   ) {
     let v = T::cast_from(128u32 << (bit_depth - 8));
-    for y in 0..height {
-      for x in 0..width {
-        output[y][x] = v;
-      }
+    for line in output.rows_iter_mut().take(height) {
+      line[..width].fill(v);
     }
   }
 
@@ -755,7 +821,7 @@ pub(crate) mod rust {
     });
     let avg = T::cast_from((sum + (height >> 1) as u32) / height as u32);
     for line in output.rows_iter_mut().take(height) {
-      line[..width].iter_mut().for_each(|v| *v = avg);
+      line[..width].fill(avg);
     }
   }
 
@@ -769,7 +835,7 @@ pub(crate) mod rust {
     });
     let avg = T::cast_from((sum + (width >> 1) as u32) / width as u32);
     for line in output.rows_iter_mut().take(height) {
-      line[..width].iter_mut().for_each(|v| *v = avg);
+      line[..width].fill(avg);
     }
   }
 
@@ -778,9 +844,7 @@ pub(crate) mod rust {
     height: usize,
   ) {
     for (line, l) in output.rows_iter_mut().zip(left[..height].iter().rev()) {
-      for v in &mut line[..width] {
-        *v = *l;
-      }
+      line[..width].fill(*l);
     }
   }
 
@@ -789,7 +853,7 @@ pub(crate) mod rust {
     height: usize,
   ) {
     for line in output.rows_iter_mut().take(height) {
-      line[..width].clone_from_slice(&above[..width])
+      line[..width].copy_from_slice(&above[..width])
     }
   }
 
@@ -953,6 +1017,51 @@ pub(crate) mod rust {
     }
   }
 
+  pub(crate) fn pred_cfl_ac<T: Pixel, const XDEC: usize, const YDEC: usize>(
+    ac: &mut [MaybeUninit<i16>], luma: &PlaneRegion<'_, T>,
+    plane_bsize: BlockSize, w_pad: usize, h_pad: usize, _cpu: CpuFeatureLevel,
+  ) {
+    let max_luma_w = (plane_bsize.width() - w_pad * 4) << XDEC;
+    let max_luma_h = (plane_bsize.height() - h_pad * 4) << YDEC;
+    let max_luma_x: usize = max_luma_w.max(8) - (1 << XDEC);
+    let max_luma_y: usize = max_luma_h.max(8) - (1 << YDEC);
+    let mut sum: i32 = 0;
+
+    let ac = &mut ac[..plane_bsize.area()];
+
+    for (sub_y, ac_rows) in
+      ac.chunks_exact_mut(plane_bsize.width()).enumerate()
+    {
+      for (sub_x, ac_item) in ac_rows.iter_mut().enumerate() {
+        // Refer to https://aomediacodec.github.io/av1-spec/#predict-chroma-from-luma-process
+        let luma_y = sub_y << YDEC;
+        let luma_x = sub_x << XDEC;
+        let y = luma_y.min(max_luma_y);
+        let x = luma_x.min(max_luma_x);
+        let mut sample: i16 = i16::cast_from(luma[y][x]);
+        if XDEC != 0 {
+          sample += i16::cast_from(luma[y][x + 1]);
+        }
+        if YDEC != 0 {
+          debug_assert!(XDEC != 0);
+          sample += i16::cast_from(luma[y + 1][x])
+            + i16::cast_from(luma[y + 1][x + 1]);
+        }
+        sample <<= 3 - XDEC - YDEC;
+        ac_item.write(sample);
+        sum += sample as i32;
+      }
+    }
+    // SAFETY: the loop above has initialized all items
+    let ac = unsafe { slice_assume_init_mut(ac) };
+    let shift = plane_bsize.width_log2() + plane_bsize.height_log2();
+    let average = ((sum + (1 << (shift - 1))) >> shift) as i16;
+
+    for val in ac {
+      *val -= average;
+    }
+  }
+
   pub(crate) fn pred_cfl_inner<T: Pixel>(
     output: &mut PlaneRegionMut<'_, T>, ac: &[i16], alpha: i16, width: usize,
     height: usize, bit_depth: usize,
@@ -960,8 +1069,7 @@ pub(crate) mod rust {
     if alpha == 0 {
       return;
     }
-    assert!(32 >= width);
-    assert!(ac.len() >= 32 * (height - 1) + width);
+    debug_assert!(ac.len() >= width * height);
     assert!(output.plane_cfg.stride >= width);
     assert!(output.rows_iter().len() >= height);
 
@@ -969,11 +1077,11 @@ pub(crate) mod rust {
     let avg: i32 = output[0][0].into();
 
     for (line, luma) in
-      output.rows_iter_mut().zip(ac.chunks(width)).take(height)
+      output.rows_iter_mut().zip(ac.chunks_exact(width)).take(height)
     {
       for (v, &l) in line[..width].iter_mut().zip(luma[..width].iter()) {
         *v = T::cast_from(
-          (avg + get_scaled_luma_q0(alpha, l)).max(0).min(sample_max),
+          (avg + get_scaled_luma_q0(alpha, l)).clamp(0, sample_max),
         );
       }
     }
@@ -1011,152 +1119,191 @@ pub(crate) mod rust {
     pred_cfl_inner(output, ac, alpha, width, height, bit_depth);
   }
 
-  #[allow(clippy::clone_double_ref)]
+  #[allow(clippy::collapsible_if)]
+  #[allow(clippy::collapsible_else_if)]
+  #[allow(clippy::needless_return)]
+  pub(crate) const fn select_ief_strength(
+    width: usize, height: usize, smooth_filter: bool, angle_delta: isize,
+  ) -> u8 {
+    let block_wh = width + height;
+    let abs_delta = angle_delta.unsigned_abs();
+
+    if smooth_filter {
+      if block_wh <= 8 {
+        if abs_delta >= 64 {
+          return 2;
+        }
+        if abs_delta >= 40 {
+          return 1;
+        }
+      } else if block_wh <= 16 {
+        if abs_delta >= 48 {
+          return 2;
+        }
+        if abs_delta >= 20 {
+          return 1;
+        }
+      } else if block_wh <= 24 {
+        if abs_delta >= 4 {
+          return 3;
+        }
+      } else {
+        return 3;
+      }
+    } else {
+      if block_wh <= 8 {
+        if abs_delta >= 56 {
+          return 1;
+        }
+      } else if block_wh <= 16 {
+        if abs_delta >= 40 {
+          return 1;
+        }
+      } else if block_wh <= 24 {
+        if abs_delta >= 32 {
+          return 3;
+        }
+        if abs_delta >= 16 {
+          return 2;
+        }
+        if abs_delta >= 8 {
+          return 1;
+        }
+      } else if block_wh <= 32 {
+        if abs_delta >= 32 {
+          return 3;
+        }
+        if abs_delta >= 4 {
+          return 2;
+        }
+        return 1;
+      } else {
+        return 3;
+      }
+    }
+
+    return 0;
+  }
+
+  pub(crate) const fn select_ief_upsample(
+    width: usize, height: usize, smooth_filter: bool, angle_delta: isize,
+  ) -> bool {
+    let block_wh = width + height;
+    let abs_delta = angle_delta.unsigned_abs();
+
+    if abs_delta == 0 || abs_delta >= 40 {
+      false
+    } else if smooth_filter {
+      block_wh <= 8
+    } else {
+      block_wh <= 16
+    }
+  }
+
+  pub(crate) fn filter_edge<T: Pixel>(
+    size: usize, strength: u8, edge: &mut [T],
+  ) {
+    const INTRA_EDGE_KERNEL: [[u32; 5]; 3] =
+      [[0, 4, 8, 4, 0], [0, 5, 6, 5, 0], [2, 4, 4, 4, 2]];
+
+    if strength == 0 {
+      return;
+    }
+
+    // Copy the edge buffer to avoid predicting from
+    // just-filtered samples.
+    let mut edge_filtered = [MaybeUninit::<T>::uninit(); MAX_TX_SIZE * 4 + 1];
+    let edge_filtered =
+      init_slice_repeat_mut(&mut edge_filtered[..edge.len()], T::zero());
+    edge_filtered.copy_from_slice(&edge[..edge.len()]);
+
+    for i in 1..size {
+      let mut s = 0;
+
+      for j in 0..INTRA_EDGE_KERNEL[0].len() {
+        let k = (i + j).saturating_sub(2).min(size - 1);
+        s += INTRA_EDGE_KERNEL[(strength - 1) as usize][j]
+          * edge[k].to_u32().unwrap();
+      }
+
+      edge_filtered[i] = T::cast_from((s + 8) >> 4);
+    }
+    edge.copy_from_slice(edge_filtered);
+  }
+
+  pub(crate) fn upsample_edge<T: Pixel>(
+    size: usize, edge: &mut [T], bit_depth: usize,
+  ) {
+    // The input edge should be valid in the -1..size range,
+    // where the -1 index is the top-left edge pixel. Since
+    // negative indices are unsafe in Rust, the caller is
+    // expected to globally offset it by 1, which makes the
+    // input range 0..=size.
+    let mut dup = [MaybeUninit::<T>::uninit(); MAX_TX_SIZE];
+    let dup = init_slice_repeat_mut(&mut dup[..size + 3], T::zero());
+    dup[0] = edge[0];
+    dup[1..=size + 1].copy_from_slice(&edge[0..=size]);
+    dup[size + 2] = edge[size];
+
+    // Past here the edge is being filtered, and its
+    // effective range is shifted from -1..size to
+    // -2..2*size-1. Again, because this is safe Rust,
+    // we cannot use negative indices, and the actual range
+    // will be 0..=2*size. The caller is expected to adjust
+    // its indices on receipt of the filtered edge.
+    edge[0] = dup[0];
+
+    for i in 0..size {
+      let mut s = -dup[i].to_i32().unwrap()
+        + (9 * dup[i + 1].to_i32().unwrap())
+        + (9 * dup[i + 2].to_i32().unwrap())
+        - dup[i + 3].to_i32().unwrap();
+      s = ((s + 8) / 16).clamp(0, (1 << bit_depth) - 1);
+
+      edge[2 * i + 1] = T::cast_from(s);
+      edge[2 * i + 2] = dup[i + 2];
+    }
+  }
+
+  pub(crate) const fn dr_intra_derivative(p_angle: usize) -> usize {
+    match p_angle {
+      3 => 1023,
+      6 => 547,
+      9 => 372,
+      14 => 273,
+      17 => 215,
+      20 => 178,
+      23 => 151,
+      26 => 132,
+      29 => 116,
+      32 => 102,
+      36 => 90,
+      39 => 80,
+      42 => 71,
+      45 => 64,
+      48 => 57,
+      51 => 51,
+      54 => 45,
+      58 => 40,
+      61 => 35,
+      64 => 31,
+      67 => 27,
+      70 => 23,
+      73 => 19,
+      76 => 15,
+      81 => 11,
+      84 => 7,
+      87 => 3,
+      _ => 0,
+    }
+  }
+
   pub(crate) fn pred_directional<T: Pixel>(
     output: &mut PlaneRegionMut<'_, T>, above: &[T], left: &[T],
     top_left: &[T], p_angle: usize, width: usize, height: usize,
     bit_depth: usize, ief_params: Option<IntraEdgeFilterParameters>,
   ) {
-    #[allow(clippy::collapsible_if)]
-    #[allow(clippy::collapsible_else_if)]
-    #[allow(clippy::needless_return)]
-    fn select_ief_strength(
-      width: usize, height: usize, smooth_filter: bool, angle_delta: isize,
-    ) -> u8 {
-      let block_wh = width + height;
-      let abs_delta = angle_delta.abs() as usize;
-
-      if smooth_filter {
-        if block_wh <= 8 {
-          if abs_delta >= 64 {
-            return 2;
-          }
-          if abs_delta >= 40 {
-            return 1;
-          }
-        } else if block_wh <= 16 {
-          if abs_delta >= 48 {
-            return 2;
-          }
-          if abs_delta >= 20 {
-            return 1;
-          }
-        } else if block_wh <= 24 {
-          if abs_delta >= 4 {
-            return 3;
-          }
-        } else {
-          return 3;
-        }
-      } else {
-        if block_wh <= 8 {
-          if abs_delta >= 56 {
-            return 1;
-          }
-        } else if block_wh <= 16 {
-          if abs_delta >= 40 {
-            return 1;
-          }
-        } else if block_wh <= 24 {
-          if abs_delta >= 32 {
-            return 3;
-          }
-          if abs_delta >= 16 {
-            return 2;
-          }
-          if abs_delta >= 8 {
-            return 1;
-          }
-        } else if block_wh <= 32 {
-          if abs_delta >= 32 {
-            return 3;
-          }
-          if abs_delta >= 4 {
-            return 2;
-          }
-          return 1;
-        } else {
-          return 3;
-        }
-      }
-
-      return 0;
-    }
-
-    fn select_ief_upsample(
-      width: usize, height: usize, smooth_filter: bool, angle_delta: isize,
-    ) -> bool {
-      let block_wh = width + height;
-      let abs_delta = angle_delta.abs() as usize;
-
-      if abs_delta == 0 || abs_delta >= 40 {
-        false
-      } else if smooth_filter {
-        block_wh <= 8
-      } else {
-        block_wh <= 16
-      }
-    }
-
-    fn filter_edge<T: Pixel>(size: usize, strength: u8, edge: &mut [T]) {
-      const INTRA_EDGE_KERNEL: [[u32; 5]; 3] =
-        [[0, 4, 8, 4, 0], [0, 5, 6, 5, 0], [2, 4, 4, 4, 2]];
-
-      if strength == 0 {
-        return;
-      }
-
-      // Copy the edge buffer to avoid predicting from
-      // just-filtered samples.
-      let mut edge_filtered = vec![T::cast_from(0); edge.len()];
-      edge_filtered.copy_from_slice(&edge[..edge.len()]);
-
-      for i in 1..size {
-        let mut s = 0;
-
-        for j in 0..INTRA_EDGE_KERNEL[0].len() {
-          let k = (i + j).saturating_sub(2).min(size - 1);
-          s += INTRA_EDGE_KERNEL[(strength - 1) as usize][j]
-            * edge[k].to_u32().unwrap();
-        }
-
-        edge_filtered[i] = T::cast_from((s + 8) >> 4);
-      }
-      edge.copy_from_slice(&edge_filtered[..]);
-    }
-
-    fn upsample_edge<T: Pixel>(size: usize, edge: &mut [T], bit_depth: usize) {
-      // The input edge should be valid in the -1..size range,
-      // where the -1 index is the top-left edge pixel. Since
-      // negative indices are unsafe in Rust, the caller is
-      // expected to globally offset it by 1, which makes the
-      // input range 0..=size.
-      let mut dup = vec![T::cast_from(0); size + 3];
-      dup[0] = edge[0];
-      dup[1..=size + 1].copy_from_slice(&edge[0..=size]);
-      dup[size + 2] = edge[size];
-
-      // Past here the edge is being filtered, and its
-      // effective range is shifted from -1..size to
-      // -2..2*size-1. Again, because this is safe Rust,
-      // we cannot use negative indices, and the actual range
-      // will be 0..=2*size. The caller is expected to adjust
-      // its indices on receipt of the filtered edge.
-      edge[0] = dup[0];
-
-      for i in 0..size {
-        let mut s = -dup[i].to_i32().unwrap()
-          + (9 * dup[i + 1].to_i32().unwrap())
-          + (9 * dup[i + 2].to_i32().unwrap())
-          - dup[i + 3].to_i32().unwrap();
-        s = (((s + 8) / 16) as i32).max(0).min((1 << bit_depth) - 1);
-
-        edge[2 * i + 1] = T::cast_from(s);
-        edge[2 * i + 2] = dup[i + 2];
-      }
-    }
-
-    let sample_max = ((1 << bit_depth) - 1) as i32;
+    let sample_max = (1 << bit_depth) - 1;
 
     let max_x = output.plane_cfg.width as isize - 1;
     let max_y = output.plane_cfg.height as isize - 1;
@@ -1172,14 +1319,22 @@ pub(crate) mod rust {
 
     // Initialize above and left edge buffers of the largest possible needed size if upsampled
     // The first value is the top left pixel, also mutable and indexed at -1 in the spec
-    let mut above_filtered: Vec<T> =
-      vec![T::cast_from(0); (width + height) * 2 + 1];
-    let mut left_filtered: Vec<T> =
-      vec![T::cast_from(0); (width + height) * 2 + 1];
+    let mut above_filtered = [MaybeUninit::<T>::uninit(); MAX_TX_SIZE * 4 + 1];
+    let above_filtered = init_slice_repeat_mut(
+      &mut above_filtered[..=(width + height) * 2],
+      T::zero(),
+    );
+    let mut left_filtered = [MaybeUninit::<T>::uninit(); MAX_TX_SIZE * 4 + 1];
+    let left_filtered = init_slice_repeat_mut(
+      &mut left_filtered[..=(width + height) * 2],
+      T::zero(),
+    );
 
     if enable_edge_filter {
-      above_filtered[1..=above.len()].clone_from_slice(above);
-      for i in 1..=left.len() {
+      let above_len = above.len().min(above_filtered.len() - 1);
+      let left_len = left.len().min(left_filtered.len() - 1);
+      above_filtered[1..=above_len].clone_from_slice(&above[..above_len]);
+      for i in 1..=left_len {
         left_filtered[i] = left[left.len() - i];
       }
 
@@ -1198,24 +1353,20 @@ pub(crate) mod rust {
             + 1, // left
         );
 
-        if output.rect().y > 0 {
-          let filter_strength = select_ief_strength(
-            width,
-            height,
-            smooth_filter,
-            p_angle as isize - 90,
-          );
-          filter_edge(num_px.0, filter_strength, &mut above_filtered[..]);
-        }
-        if output.rect().x > 0 {
-          let filter_strength = select_ief_strength(
-            width,
-            height,
-            smooth_filter,
-            p_angle as isize - 180,
-          );
-          filter_edge(num_px.1, filter_strength, &mut left_filtered[..]);
-        }
+        let filter_strength = select_ief_strength(
+          width,
+          height,
+          smooth_filter,
+          p_angle as isize - 90,
+        );
+        filter_edge(num_px.0, filter_strength, above_filtered);
+        let filter_strength = select_ief_strength(
+          width,
+          height,
+          smooth_filter,
+          p_angle as isize - 180,
+        );
+        filter_edge(num_px.1, filter_strength, left_filtered);
       }
 
       let num_px = (
@@ -1230,7 +1381,7 @@ pub(crate) mod rust {
         p_angle as isize - 90,
       );
       if upsample_above {
-        upsample_edge(num_px.0, &mut above_filtered[..], bit_depth);
+        upsample_edge(num_px.0, above_filtered, bit_depth);
       }
       upsample_left = select_ief_upsample(
         width,
@@ -1239,45 +1390,12 @@ pub(crate) mod rust {
         p_angle as isize - 180,
       );
       if upsample_left {
-        upsample_edge(num_px.1, &mut left_filtered[..], bit_depth);
+        upsample_edge(num_px.1, left_filtered, bit_depth);
       }
 
       left_filtered.reverse();
-      above_edge = &above_filtered[..];
-      left_edge = &left_filtered[..];
-    }
-
-    fn dr_intra_derivative(p_angle: usize) -> usize {
-      match p_angle {
-        3 => 1023,
-        6 => 547,
-        9 => 372,
-        14 => 273,
-        17 => 215,
-        20 => 178,
-        23 => 151,
-        26 => 132,
-        29 => 116,
-        32 => 102,
-        36 => 90,
-        39 => 80,
-        42 => 71,
-        45 => 64,
-        48 => 57,
-        51 => 51,
-        54 => 45,
-        58 => 40,
-        61 => 35,
-        64 => 31,
-        67 => 27,
-        70 => 23,
-        73 => 19,
-        76 => 15,
-        81 => 11,
-        84 => 7,
-        87 => 3,
-        _ => 0,
-      }
+      above_edge = above_filtered;
+      left_edge = left_filtered;
     }
 
     let dx = if p_angle < 90 {
@@ -1311,16 +1429,15 @@ pub(crate) mod rust {
           let base = (idx >> (6 - upsample_above)) + (j << upsample_above);
           let shift = (((idx << upsample_above) >> 1) & 31) as i32;
           let max_base_x = (height + width - 1) << upsample_above;
-          let v = if base < max_base_x {
+          let v = (if base < max_base_x {
             let a: i32 = above_edge[base + offset_above].into();
             let b: i32 = above_edge[base + 1 + offset_above].into();
             round_shift(a * (32 - shift) + b * shift, 5)
           } else {
             let c: i32 = above_edge[max_base_x + offset_above].into();
             c
-          }
-          .max(0)
-          .min(sample_max);
+          })
+          .clamp(0, sample_max);
           row[j] = T::cast_from(v);
         }
       }
@@ -1341,8 +1458,7 @@ pub(crate) mod rust {
             let b: i32 =
               above_edge[(base + 1 + offset_above as isize) as usize].into();
             let v = round_shift(a * (32 - shift) + b * shift, 5)
-              .max(0)
-              .min(sample_max);
+              .clamp(0, sample_max);
             row[j] = T::cast_from(v);
           } else {
             let idx = (i << 6) as isize - ((j + 1) * dy) as isize;
@@ -1364,8 +1480,7 @@ pub(crate) mod rust {
             }
             .into();
             let v = round_shift(a * (32 - shift) + b * shift, 5)
-              .max(0)
-              .min(sample_max);
+              .clamp(0, sample_max);
             row[j] = T::cast_from(v);
           }
         }
@@ -1381,9 +1496,8 @@ pub(crate) mod rust {
           let a: i32 = left_edge[l.saturating_sub(base + offset_left)].into();
           let b: i32 =
             left_edge[l.saturating_sub(base + offset_left + 1)].into();
-          let v = round_shift(a * (32 - shift) + b * shift, 5)
-            .max(0)
-            .min(sample_max);
+          let v =
+            round_shift(a * (32 - shift) + b * shift, 5).clamp(0, sample_max);
           row[j] = T::cast_from(v);
         }
       }
@@ -1399,14 +1513,10 @@ mod test {
 
   #[test]
   fn pred_matches_u8() {
-    let mut edge_buf: Aligned<[u8; 2 * MAX_TX_SIZE + 1]> =
-      Aligned::uninitialized();
-    for i in 0..edge_buf.data.len() {
-      edge_buf.data[i] = (i + 32).saturating_sub(MAX_TX_SIZE).as_();
-    }
-    let left = &edge_buf.data[MAX_TX_SIZE - 4..MAX_TX_SIZE];
-    let above = &edge_buf.data[MAX_TX_SIZE + 1..MAX_TX_SIZE + 5];
-    let top_left = edge_buf.data[MAX_TX_SIZE];
+    let edge_buf =
+      Aligned::from_fn(|i| (i + 32).saturating_sub(MAX_TX_SIZE * 2).as_());
+    let (all_left, top_left, above) = IntraEdge::mock(&edge_buf).as_slices();
+    let left = &all_left[all_left.len() - 4..];
 
     let mut output = Plane::from_slice(&[0u8; 4 * 4], 4);
 
@@ -1434,7 +1544,7 @@ mod test {
       [31, 31, 31, 31, 30, 30, 30, 30, 29, 29, 29, 29, 28, 28, 28, 28]
     );
 
-    pred_paeth(&mut output.as_region_mut(), above, left, top_left, 4, 4);
+    pred_paeth(&mut output.as_region_mut(), above, left, top_left[0], 4, 4);
     assert_eq!(
       &output.data[..],
       [32, 34, 35, 36, 30, 32, 32, 36, 29, 32, 32, 32, 28, 28, 32, 32]
@@ -1458,9 +1568,7 @@ mod test {
       [33, 34, 35, 36, 31, 31, 32, 33, 30, 30, 30, 31, 29, 30, 30, 30]
     );
 
-    let left = &edge_buf.data[MAX_TX_SIZE - 8..MAX_TX_SIZE];
-    let above = &edge_buf.data[MAX_TX_SIZE + 1..MAX_TX_SIZE + 9];
-    let top_left = &edge_buf.data[MAX_TX_SIZE..=MAX_TX_SIZE];
+    let left = &all_left[all_left.len() - 8..];
     let angles = [
       3, 6, 9, 14, 17, 20, 23, 26, 29, 32, 36, 39, 42, 45, 48, 51, 54, 58, 61,
       64, 67, 70, 73, 76, 81, 84, 87,
@@ -1542,7 +1650,7 @@ mod test {
       }
     }
 
-    let above_left = unsafe { *above.as_ptr().offset(-1) };
+    let above_left = max12bit;
 
     pred_paeth(
       &mut o.as_region_mut(),
