@@ -1,5 +1,5 @@
 // Copyright (c) 2001-2016, Alliance for Open Media. All rights reserved
-// Copyright (c) 2017-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2017-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -10,6 +10,12 @@
 
 #![allow(non_camel_case_types)]
 
+use std::fmt;
+use std::mem::MaybeUninit;
+
+use arrayvec::*;
+use itertools::izip;
+
 use crate::api::*;
 use crate::cdef::*;
 use crate::context::*;
@@ -19,36 +25,31 @@ use crate::dist::*;
 use crate::ec::{Writer, WriterCounter, OD_BITRES};
 use crate::encode_block_with_modes;
 use crate::encoder::{FrameInvariants, IMPORTANCE_BLOCK_SIZE};
-use crate::frame::Frame;
 use crate::frame::*;
 use crate::header::ReferenceMode;
 use crate::lrf::*;
-use crate::luma_ac;
 use crate::mc::MotionVector;
-use crate::me::*;
+use crate::me::estimate_motion;
+use crate::me::MVSamplingMode;
+use crate::me::MotionSearchResult;
 use crate::motion_compensate;
+use crate::partition::PartitionType::*;
 use crate::partition::RefType::*;
 use crate::partition::*;
 use crate::predict::{
-  AngleDelta, IntraEdgeFilterParameters, IntraParam, PredictionMode,
+  luma_ac, AngleDelta, IntraEdgeFilterParameters, IntraParam, PredictionMode,
   RAV1E_INTER_COMPOUND_MODES, RAV1E_INTER_MODES_MINIMAL, RAV1E_INTRA_MODES,
 };
 use crate::rdo_tables::*;
 use crate::tiling::*;
 use crate::transform::{TxSet, TxSize, TxType, RAV1E_TX_TYPES};
-use crate::util::{init_slice_repeat_mut, Aligned, CastFromPrimitive, Pixel};
+use crate::util::{init_slice_repeat_mut, Aligned, Pixel};
 use crate::write_tx_blocks;
 use crate::write_tx_tree;
 use crate::Tune;
 use crate::{encode_block_post_cdef, encode_block_pre_cdef};
 
-use crate::partition::PartitionType::*;
-use arrayvec::*;
-use itertools::izip;
-use std::fmt;
-use std::mem::MaybeUninit;
-
-#[derive(Copy, Clone, PartialEq)]
+#[derive(Copy, Clone, PartialEq, Eq)]
 pub enum RDOType {
   PixelDistRealRate,
   TxDistRealRate,
@@ -57,7 +58,7 @@ pub enum RDOType {
 
 impl RDOType {
   #[inline]
-  pub fn needs_tx_dist(self) -> bool {
+  pub const fn needs_tx_dist(self) -> bool {
     match self {
       // Pixel-domain distortion and exact ec rate
       RDOType::PixelDistRealRate => false,
@@ -68,7 +69,7 @@ impl RDOType {
     }
   }
   #[inline]
-  pub fn needs_coeff_rate(self) -> bool {
+  pub const fn needs_coeff_rate(self) -> bool {
     match self {
       RDOType::PixelDistRealRate => true,
       RDOType::TxDistRealRate => true,
@@ -105,9 +106,9 @@ pub struct PartitionParameters {
 impl Default for PartitionParameters {
   fn default() -> Self {
     PartitionParameters {
-      rd_cost: std::f64::MAX,
+      rd_cost: f64::MAX,
       bo: TileBlockOffset::default(),
-      bsize: BlockSize::BLOCK_INVALID,
+      bsize: BlockSize::BLOCK_32X32,
       pred_mode_luma: PredictionMode::default(),
       pred_mode_chroma: PredictionMode::default(),
       pred_cfl_params: CFLParams::default(),
@@ -137,104 +138,31 @@ pub fn estimate_rate(qindex: u8, ts: TxSize, fast_distortion: u64) -> u64 {
   (y0 + (((fast_distortion as i64 - x0) * slope) >> 8)).max(0) as u64
 }
 
-// The microbenchmarks perform better with inlining turned off
-#[inline(never)]
-fn cdef_dist_wxh_8x8<T: Pixel>(
-  src1: &PlaneRegion<'_, T>, src2: &PlaneRegion<'_, T>, bit_depth: usize,
-) -> RawDistortion {
-  debug_assert!(src1.plane_cfg.xdec == 0);
-  debug_assert!(src1.plane_cfg.ydec == 0);
-  debug_assert!(src2.plane_cfg.xdec == 0);
-  debug_assert!(src2.plane_cfg.ydec == 0);
-
-  // Sum into columns to improve auto-vectorization
-  let mut sum_s_cols: [u16; 8] = [0; 8];
-  let mut sum_d_cols: [u16; 8] = [0; 8];
-  let mut sum_s2_cols: [u32; 8] = [0; 8];
-  let mut sum_d2_cols: [u32; 8] = [0; 8];
-  let mut sum_sd_cols: [u32; 8] = [0; 8];
-
-  // Check upfront that 8 rows are available.
-  let _row1 = &src1[7];
-  let _row2 = &src2[7];
-
-  for j in 0..8 {
-    let row1 = &src1[j][0..8];
-    let row2 = &src2[j][0..8];
-    for (sum_s, sum_d, sum_s2, sum_d2, sum_sd, s, d) in izip!(
-      &mut sum_s_cols,
-      &mut sum_d_cols,
-      &mut sum_s2_cols,
-      &mut sum_d2_cols,
-      &mut sum_sd_cols,
-      row1,
-      row2
-    ) {
-      // Don't convert directly to u32 to allow better vectorization
-      let s: u16 = u16::cast_from(*s);
-      let d: u16 = u16::cast_from(*d);
-      *sum_s += s;
-      *sum_d += d;
-
-      // Convert to u32 to avoid overflows when multiplying
-      let s: u32 = s as u32;
-      let d: u32 = d as u32;
-
-      *sum_s2 += s * s;
-      *sum_d2 += d * d;
-      *sum_sd += s * d;
-    }
-  }
-
-  // Sum together the sum of columns
-  let sum_s: i64 =
-    sum_s_cols.iter().map(|&a| u32::cast_from(a)).sum::<u32>() as i64;
-  let sum_d: i64 =
-    sum_d_cols.iter().map(|&a| u32::cast_from(a)).sum::<u32>() as i64;
-  let sum_s2: i64 = sum_s2_cols.iter().sum::<u32>() as i64;
-  let sum_d2: i64 = sum_d2_cols.iter().sum::<u32>() as i64;
-  let sum_sd: i64 = sum_sd_cols.iter().sum::<u32>() as i64;
-
-  // Use sums to calculate distortion
-  let svar = sum_s2 - ((sum_s * sum_s + 32) >> 6);
-  let dvar = sum_d2 - ((sum_d * sum_d + 32) >> 6);
-  let sse = (sum_d2 + sum_s2 - 2 * sum_sd) as u64;
-  RawDistortion::new(ssim_boost(svar, dvar, bit_depth).mul_u64(sse))
-}
-
-#[inline(always)]
-pub fn ssim_boost(svar: i64, dvar: i64, bit_depth: usize) -> DistortionScale {
-  let coeff_shift = bit_depth - 8;
-
-  //The two constants were tuned for CDEF, but can probably be better tuned for use in general RDO
-  DistortionScale::new(
-    (4033_f64 / 16_384_f64)
-      * (svar + dvar + (16_384 << (2 * coeff_shift))) as f64
-      / f64::sqrt(((16_265_089i64 << (4 * coeff_shift)) + svar * dvar) as f64),
-  )
-}
-
 #[allow(unused)]
 pub fn cdef_dist_wxh<T: Pixel, F: Fn(Area, BlockSize) -> DistortionScale>(
   src1: &PlaneRegion<'_, T>, src2: &PlaneRegion<'_, T>, w: usize, h: usize,
-  bit_depth: usize, compute_bias: F,
+  bit_depth: usize, compute_bias: F, cpu: CpuFeatureLevel,
 ) -> Distortion {
-  assert!(w & 0x7 == 0);
-  assert!(h & 0x7 == 0);
   debug_assert!(src1.plane_cfg.xdec == 0);
   debug_assert!(src1.plane_cfg.ydec == 0);
   debug_assert!(src2.plane_cfg.xdec == 0);
   debug_assert!(src2.plane_cfg.ydec == 0);
 
   let mut sum = Distortion::zero();
-  for j in 0isize..h as isize / 8 {
-    for i in 0isize..w as isize / 8 {
-      let area = Area::StartingAt { x: i * 8, y: j * 8 };
-      let value = cdef_dist_wxh_8x8(
+  for y in (0..h).step_by(8) {
+    for x in (0..w).step_by(8) {
+      let kernel_h = (h - y).min(8);
+      let kernel_w = (w - x).min(8);
+      let area = Area::StartingAt { x: x as isize, y: y as isize };
+
+      let value = RawDistortion(cdef_dist_kernel(
         &src1.subregion(area),
         &src2.subregion(area),
+        kernel_w,
+        kernel_h,
         bit_depth,
-      );
+        cpu,
+      ) as u64);
 
       // cdef is always called on non-subsampled planes, so BLOCK_8X8 is
       // correct here.
@@ -261,8 +189,8 @@ pub fn sse_wxh<T: Pixel, F: Fn(Area, BlockSize) -> DistortionScale>(
 
   let imp_bsize = BlockSize::from_width_and_height(imp_block_w, imp_block_h);
 
-  let n_imp_blocks_w = (w + CHUNK_SIZE - 1) / CHUNK_SIZE;
-  let n_imp_blocks_h = (h + CHUNK_SIZE - 1) / CHUNK_SIZE;
+  let n_imp_blocks_w = w.div_ceil(CHUNK_SIZE);
+  let n_imp_blocks_h = h.div_ceil(CHUNK_SIZE);
 
   // TODO: Copying biases into a buffer is slow. It would be best if biases were
   // passed directly. To do this, we would need different versions of the
@@ -295,7 +223,9 @@ pub fn sse_wxh<T: Pixel, F: Fn(Area, BlockSize) -> DistortionScale>(
   ))
 }
 
-pub fn clip_visible_bsize(
+// TODO consider saturating_sub later
+#[allow(clippy::implicit_saturating_sub)]
+pub const fn clip_visible_bsize(
   frame_w: usize, frame_h: usize, bsize: BlockSize, x: usize, y: usize,
 ) -> (usize, usize) {
   let blk_w = bsize.width();
@@ -344,71 +274,22 @@ fn compute_distortion<T: Pixel>(
   }
 
   let mut distortion = match fi.config.tune {
-    Tune::Psychovisual if bsize.width() >= 8 && bsize.height() >= 8 => {
-      let w8 = visible_w & !7;
-      let h8 = visible_h & !7;
-      let mut sum = Distortion(0);
-      if w8 > 0 && h8 > 0 {
-        sum += cdef_dist_wxh(
-          &input_region,
-          &rec_region,
-          w8,
-          h8,
-          fi.sequence.bit_depth,
-          |bias_area, bsize| {
-            distortion_scale(
-              fi,
-              input_region.subregion(bias_area).frame_block_offset(),
-              bsize,
-            )
-          },
-        );
-      }
-      if visible_w > w8 && h8 > 0 {
-        let area = Area::StartingAt { x: w8 as isize, y: 0 };
-        sum += sse_wxh(
-          &input_region.subregion(area),
-          &rec_region.subregion(area),
-          visible_w - w8,
-          h8,
-          |bias_area, bsize| {
-            spatiotemporal_scale(
-              fi,
-              input_region
-                .subregion(area)
-                .subregion(bias_area)
-                .frame_block_offset(),
-              bsize,
-            )
-          },
-          fi.sequence.bit_depth,
-          fi.cpu_feature_level,
-        );
-      }
-      if visible_h > h8 && visible_w > 0 {
-        let area = Area::StartingAt { x: 0, y: h8 as isize };
-        sum += sse_wxh(
-          &input_region.subregion(area),
-          &rec_region.subregion(area),
-          visible_w,
-          visible_h - h8,
-          |bias_area, bsize| {
-            spatiotemporal_scale(
-              fi,
-              input_region
-                .subregion(area)
-                .subregion(bias_area)
-                .frame_block_offset(),
-              bsize,
-            )
-          },
-          fi.sequence.bit_depth,
-          fi.cpu_feature_level,
-        );
-      }
-      sum
-    }
-    Tune::Psnr | Tune::Psychovisual => sse_wxh(
+    Tune::Psychovisual => cdef_dist_wxh(
+      &input_region,
+      &rec_region,
+      visible_w,
+      visible_h,
+      fi.sequence.bit_depth,
+      |bias_area, bsize| {
+        distortion_scale(
+          fi,
+          input_region.subregion(bias_area).frame_block_offset(),
+          bsize,
+        )
+      },
+      fi.cpu_feature_level,
+    ),
+    Tune::Psnr => sse_wxh(
       &input_region,
       &rec_region,
       visible_w,
@@ -554,6 +435,11 @@ fn compute_tx_distortion<T: Pixel>(
 
 /// Compute a scaling factor to multiply the distortion of a block by,
 /// this factor is determined using temporal RDO.
+///
+/// # Panics
+///
+/// - If called with `bsize` of 8x8 or smaller
+/// - If the coded frame data doesn't exist on the `FrameInvariants`
 pub fn distortion_scale<T: Pixel>(
   fi: &FrameInvariants<T>, frame_bo: PlaneBlockOffset, bsize: BlockSize,
 ) -> DistortionScale {
@@ -568,9 +454,13 @@ pub fn distortion_scale<T: Pixel>(
   let x = frame_bo.0.x >> IMPORTANCE_BLOCK_TO_BLOCK_SHIFT;
   let y = frame_bo.0.y >> IMPORTANCE_BLOCK_TO_BLOCK_SHIFT;
 
-  fi.distortion_scales[y * fi.w_in_imp_b + x]
+  let coded_data = fi.coded_frame_data.as_ref().unwrap();
+  coded_data.distortion_scales[y * coded_data.w_in_imp_b + x]
 }
 
+/// # Panics
+///
+/// - If the coded frame data doesn't exist on the `FrameInvariants`
 pub fn spatiotemporal_scale<T: Pixel>(
   fi: &FrameInvariants<T>, frame_bo: PlaneBlockOffset, bsize: BlockSize,
 ) -> DistortionScale {
@@ -578,20 +468,37 @@ pub fn spatiotemporal_scale<T: Pixel>(
     return DistortionScale::default();
   }
 
+  let coded_data = fi.coded_frame_data.as_ref().unwrap();
+
   let x0 = frame_bo.0.x >> IMPORTANCE_BLOCK_TO_BLOCK_SHIFT;
   let y0 = frame_bo.0.y >> IMPORTANCE_BLOCK_TO_BLOCK_SHIFT;
-  let x1 = (x0 + bsize.width_imp_b()).min(fi.w_in_imp_b);
-  let y1 = (y0 + bsize.height_imp_b()).min(fi.h_in_imp_b);
+  let x1 = (x0 + bsize.width_imp_b()).min(coded_data.w_in_imp_b);
+  let y1 = (y0 + bsize.height_imp_b()).min(coded_data.h_in_imp_b);
   let den = (((x1 - x0) * (y1 - y0)) as u64) << DistortionScale::SHIFT;
 
+  // calling this on each slice individually improves autovectorization
+  // compared to using `Iterator::take`
+  #[inline(always)]
+  fn take_slice<T>(slice: &[T], n: usize) -> &[T] {
+    slice.get(..n).unwrap_or(slice)
+  }
+
   let mut sum = 0;
   for y in y0..y1 {
-    sum += fi.distortion_scales[y * fi.w_in_imp_b..][x0..x1]
-      .iter()
-      .zip(fi.activity_scales[y * fi.w_in_imp_b..][x0..x1].iter())
-      .take(MAX_SB_IN_IMP_B)
-      .map(|(d, a)| d.0 as u64 * a.0 as u64)
-      .sum::<u64>();
+    sum += take_slice(
+      &coded_data.distortion_scales[y * coded_data.w_in_imp_b..][x0..x1],
+      MAX_SB_IN_IMP_B,
+    )
+    .iter()
+    .zip(
+      take_slice(
+        &coded_data.activity_scales[y * coded_data.w_in_imp_b..][x0..x1],
+        MAX_SB_IN_IMP_B,
+      )
+      .iter(),
+    )
+    .map(|(d, a)| d.0 as u64 * a.0 as u64)
+    .sum::<u64>();
   }
   DistortionScale(((sum + (den >> 1)) / den) as u32)
 }
@@ -642,7 +549,7 @@ pub fn distortion_scale_for(
 
   let strength = 1.0; // empirical, see comment above
   let frac = (intra_cost + propagate_cost) / intra_cost;
-  DistortionScale::new(frac.powf(strength / 3.0))
+  frac.powf(strength / 3.0).into()
 }
 
 /// Fixed point arithmetic version of distortion scale
@@ -661,27 +568,73 @@ pub struct ScaledDistortion(u64);
 
 impl DistortionScale {
   /// Bits past the radix point
-  const SHIFT: u32 = 12;
+  const SHIFT: u32 = 14;
   /// Number of bits used. Determines the max value.
-  /// 24 bits is likely excessive.
-  const BITS: u32 = 24;
+  /// 28 bits is quite excessive.
+  const BITS: u32 = 28;
+  /// Maximum internal value
+  const MAX: u64 = (1 << Self::BITS) - 1;
 
   #[inline]
-  pub fn new(scale: f64) -> Self {
+  pub const fn new(num: u64, den: u64) -> Self {
+    let raw = (num << Self::SHIFT).saturating_add(den / 2) / den;
+    let mask = (raw <= Self::MAX) as u64;
+    Self((mask * raw + (1 - mask) * Self::MAX) as u32)
+  }
+
+  pub fn inv_mean(slice: &[Self]) -> Self {
+    use crate::util::{bexp64, blog32_q11};
+    let sum = slice.iter().map(|&s| blog32_q11(s.0) as i64).sum::<i64>();
+    let log_inv_mean_q11 =
+      (Self::SHIFT << 11) as i64 - sum / slice.len() as i64;
     Self(
-      (scale * (1 << Self::SHIFT) as f64 + 0.5)
-        .min(((1 << Self::BITS as u64) - 1) as f64) as u32,
+      bexp64((log_inv_mean_q11 + (Self::SHIFT << 11) as i64) << (57 - 11))
+        .clamp(1, (1 << Self::BITS) - 1) as u32,
     )
   }
 
+  /// Binary logarithm in Q11
+  #[inline]
+  pub const fn blog16(self) -> i16 {
+    use crate::util::blog32_q11;
+    (blog32_q11(self.0) - ((Self::SHIFT as i32) << 11)) as i16
+  }
+
+  /// Binary logarithm in Q57
+  #[inline]
+  pub const fn blog64(self) -> i64 {
+    use crate::util::{blog64, q57};
+    blog64(self.0 as i64) - q57(Self::SHIFT as i32)
+  }
+
   /// Multiply, round and shift
   /// Internal implementation, so don't use multiply trait.
   #[inline]
-  pub fn mul_u64(self, dist: u64) -> u64 {
+  pub const fn mul_u64(self, dist: u64) -> u64 {
     (self.0 as u64 * dist + (1 << Self::SHIFT >> 1)) >> Self::SHIFT
   }
 }
 
+impl std::ops::Mul for DistortionScale {
+  type Output = Self;
+
+  /// Multiply, round and shift
+  #[inline]
+  fn mul(self, rhs: Self) -> Self {
+    Self(
+      (((self.0 as u64 * rhs.0 as u64) + (1 << (Self::SHIFT - 1)))
+        >> Self::SHIFT)
+        .clamp(1, (1 << Self::BITS) - 1) as u32,
+    )
+  }
+}
+
+impl std::ops::MulAssign for DistortionScale {
+  fn mul_assign(&mut self, rhs: Self) {
+    *self = *self * rhs;
+  }
+}
+
 // Default value for DistortionScale is a fixed point 1
 impl Default for DistortionScale {
   #[inline]
@@ -696,6 +649,14 @@ impl fmt::Debug for DistortionScale {
   }
 }
 
+impl From<f64> for DistortionScale {
+  #[inline]
+  fn from(scale: f64) -> Self {
+    let den = 1 << (Self::SHIFT + 1);
+    Self::new((scale * den as f64) as u64, den)
+  }
+}
+
 impl From<DistortionScale> for f64 {
   #[inline]
   fn from(scale: DistortionScale) -> Self {
@@ -725,11 +686,11 @@ impl Distortion {
   }
 }
 
-impl std::ops::Mul<f64> for Distortion {
+impl std::ops::Mul<DistortionScale> for Distortion {
   type Output = ScaledDistortion;
   #[inline]
-  fn mul(self, rhs: f64) -> ScaledDistortion {
-    ScaledDistortion((self.0 as f64 * rhs) as u64)
+  fn mul(self, rhs: DistortionScale) -> ScaledDistortion {
+    ScaledDistortion(rhs.mul_u64(self.0))
   }
 }
 
@@ -758,7 +719,7 @@ pub fn compute_rd_cost<T: Pixel>(
   fi: &FrameInvariants<T>, rate: u32, distortion: ScaledDistortion,
 ) -> f64 {
   let rate_in_bits = (rate as f64) / ((1 << OD_BITRES) as f64);
-  distortion.0 as f64 + fi.lambda * rate_in_bits
+  fi.lambda.mul_add(rate_in_bits, distortion.0 as f64)
 }
 
 pub fn rdo_tx_size_type<T: Pixel>(
@@ -776,10 +737,11 @@ pub fn rdo_tx_size_type<T: Pixel>(
 
   let mut best_tx_type = TxType::DCT_DCT;
   let mut best_tx_size = tx_size;
-  let mut best_rd = std::f64::MAX;
+  let mut best_rd = f64::MAX;
 
-  let do_rdo_tx_size =
-    fi.tx_mode_select && fi.config.speed_settings.rdo_tx_decision && !is_inter;
+  let do_rdo_tx_size = fi.tx_mode_select
+    && fi.config.speed_settings.transform.rdo_tx_decision
+    && !is_inter;
   let rdo_tx_depth = if do_rdo_tx_size { 2 } else { 0 };
   let mut cw_checkpoint: Option<ContextWriterCheckpoint> = None;
 
@@ -787,7 +749,7 @@ pub fn rdo_tx_size_type<T: Pixel>(
     let tx_set = get_tx_set(tx_size, is_inter, fi.use_reduced_tx_set);
 
     let do_rdo_tx_type = tx_set > TxSet::TX_SET_DCTONLY
-      && fi.config.speed_settings.rdo_tx_decision
+      && fi.config.speed_settings.transform.rdo_tx_decision
       && !is_inter
       && !skip;
 
@@ -812,6 +774,7 @@ pub fn rdo_tx_size_type<T: Pixel>(
       tx_size,
       tx_set,
       tx_types,
+      best_rd,
     );
 
     if rd_cost < best_rd {
@@ -839,7 +802,7 @@ pub fn rdo_tx_size_type<T: Pixel>(
 }
 
 #[inline]
-fn dmv_in_range(mv: MotionVector, ref_mv: MotionVector) -> bool {
+const fn dmv_in_range(mv: MotionVector, ref_mv: MotionVector) -> bool {
   let diff_row = mv.row as i32 - ref_mv.row as i32;
   let diff_col = mv.col as i32 - ref_mv.col as i32;
   diff_row >= MV_LOW
@@ -849,6 +812,7 @@ fn dmv_in_range(mv: MotionVector, ref_mv: MotionVector) -> bool {
 }
 
 #[inline]
+#[profiling::function]
 fn luma_chroma_mode_rdo<T: Pixel>(
   luma_mode: PredictionMode, fi: &FrameInvariants<T>, bsize: BlockSize,
   tile_bo: TileBlockOffset, ts: &mut TileStateMut<'_, T>,
@@ -937,7 +901,7 @@ fn luma_chroma_mode_rdo<T: Pixel>(
           mv_stack,
           rdo_type,
           need_recon_pixel,
-          false,
+          None,
         );
 
         let rate = wr.tell_frac() - tell;
@@ -989,7 +953,13 @@ fn luma_chroma_mode_rdo<T: Pixel>(
   }
 }
 
-// RDO-based mode decision
+/// RDO-based mode decision
+///
+/// # Panics
+///
+/// - If the best RD found is negative.
+///   This should never happen and indicates a development error.
+#[profiling::function]
 pub fn rdo_mode_decision<T: Pixel>(
   fi: &FrameInvariants<T>, ts: &mut TileStateMut<'_, T>,
   cw: &mut ContextWriter, bsize: BlockSize, tile_bo: TileBlockOffset,
@@ -1101,7 +1071,7 @@ pub fn rdo_mode_decision<T: Pixel>(
           &[],
           rdo_type,
           true, // For CFL, luma should be always reconstructed.
-          false,
+          None,
         );
 
         let rate = wr.tell_frac() - tell;
@@ -1147,6 +1117,7 @@ pub fn rdo_mode_decision<T: Pixel>(
   }
 }
 
+#[profiling::function]
 fn inter_frame_rdo_mode_decision<T: Pixel>(
   fi: &FrameInvariants<T>, ts: &mut TileStateMut<'_, T>,
   cw: &mut ContextWriter, bsize: BlockSize, tile_bo: TileBlockOffset,
@@ -1171,10 +1142,10 @@ fn inter_frame_rdo_mode_decision<T: Pixel>(
     }
 
     if !ref_slot_set.contains(&fi.ref_frames[i.to_index()]) {
-      if fwdref == None && i.is_fwd_ref() {
+      if fwdref.is_none() && i.is_fwd_ref() {
         fwdref = Some(ref_frames_set.len());
       }
-      if bwdref == None && i.is_bwd_ref() {
+      if bwdref.is_none() && i.is_bwd_ref() {
         bwdref = Some(ref_frames_set.len());
       }
       ref_frames_set.push([i, NONE_FRAME]);
@@ -1209,8 +1180,21 @@ fn inter_frame_rdo_mode_decision<T: Pixel>(
       pmv[1] = mv_stack[1].this_mv;
     }
 
-    let res = motion_estimation(fi, ts, bsize, tile_bo, ref_frames[0], pmv);
-    let b_me = res.0;
+    let res = estimate_motion(
+      fi,
+      ts,
+      bsize.width(),
+      bsize.height(),
+      tile_bo,
+      ref_frames[0],
+      Some(pmv),
+      MVSamplingMode::CORNER { right: true, bottom: true },
+      false,
+      0,
+      None,
+    )
+    .unwrap_or_else(MotionSearchResult::empty);
+    let b_me = res.mv;
 
     mvs_from_me.push([b_me, MotionVector::default()]);
 
@@ -1223,7 +1207,7 @@ fn inter_frame_rdo_mode_decision<T: Pixel>(
     if mv_stack.len() >= 2 {
       inter_mode_set.push((PredictionMode::GLOBALMV, i));
     }
-    let include_near_mvs = fi.config.speed_settings.include_near_mvs;
+    let include_near_mvs = fi.config.speed_settings.motion.include_near_mvs;
     if include_near_mvs {
       if mv_stack.len() >= 3 {
         inter_mode_set.push((PredictionMode::NEAR1MV, i));
@@ -1271,7 +1255,9 @@ fn inter_frame_rdo_mode_decision<T: Pixel>(
         ));
         for &x in RAV1E_INTER_COMPOUND_MODES {
           // exclude any NEAR mode based on speed setting
-          if fi.config.speed_settings.include_near_mvs || !x.has_nearmv() {
+          if fi.config.speed_settings.motion.include_near_mvs
+            || !x.has_nearmv()
+          {
             let mv_stack_idx = ref_frames_set.len() - 1;
             // exclude NEAR modes if the mv_stack is too short
             if !(x.has_nearmv() && x.ref_mv_idx() >= mv_stack.len()) {
@@ -1284,7 +1270,7 @@ fn inter_frame_rdo_mode_decision<T: Pixel>(
     }
   }
 
-  let num_modes_rdo = if fi.config.speed_settings.prediction_modes
+  let num_modes_rdo = if fi.config.speed_settings.prediction.prediction_modes
     >= PredictionModesSetting::ComplexAll
   {
     inter_mode_set.len()
@@ -1359,7 +1345,8 @@ fn inter_frame_rdo_mode_decision<T: Pixel>(
       let satd = get_satd(
         &plane_org,
         &plane_ref,
-        bsize,
+        bsize.width(),
+        bsize.height(),
         fi.sequence.bit_depth,
         fi.cpu_feature_level,
       );
@@ -1403,21 +1390,21 @@ fn inter_frame_rdo_mode_decision<T: Pixel>(
   best
 }
 
+#[profiling::function]
 fn intra_frame_rdo_mode_decision<T: Pixel>(
   fi: &FrameInvariants<T>, ts: &mut TileStateMut<'_, T>,
   cw: &mut ContextWriter, bsize: BlockSize, tile_bo: TileBlockOffset,
   cw_checkpoint: &ContextWriterCheckpoint, rdo_type: RDOType,
   mut best: PartitionParameters, is_chroma_block: bool,
 ) -> PartitionParameters {
-  let num_modes_rdo: usize;
   let mut modes = ArrayVec::<_, INTRA_MODES>::new();
 
   // Reduce number of prediction modes at higher speed levels
-  num_modes_rdo = if (fi.frame_type == FrameType::KEY
-    && fi.config.speed_settings.prediction_modes
+  let num_modes_rdo = if (fi.frame_type == FrameType::KEY
+    && fi.config.speed_settings.prediction.prediction_modes
       >= PredictionModesSetting::ComplexKeyframes)
     || (fi.frame_type.has_inter()
-      && fi.config.speed_settings.prediction_modes
+      && fi.config.speed_settings.prediction.prediction_modes
         >= PredictionModesSetting::ComplexAll)
   {
     7
@@ -1447,11 +1434,13 @@ fn intra_frame_rdo_mode_decision<T: Pixel>(
     let satds = {
       // FIXME: If tx partition is used, this whole sads block should be fixed
       let tx_size = bsize.tx_size();
+      let mut edge_buf = Aligned::uninit_array();
       let edge_buf = {
         let rec = &ts.rec.planes[0].as_const();
         let po = tile_bo.plane_offset(rec.plane_cfg);
         // FIXME: If tx partition is used, get_intra_edges() should be called for each tx block
         get_intra_edges(
+          &mut edge_buf,
           rec,
           tile_bo,
           0,
@@ -1504,7 +1493,8 @@ fn intra_frame_rdo_mode_decision<T: Pixel>(
         satds_all[luma_mode as usize] = get_satd(
           &plane_org,
           &plane_ref,
-          tx_size.block_size(),
+          tx_size.width(),
+          tx_size.height(),
           fi.sequence.bit_depth,
           fi.cpu_feature_level,
         );
@@ -1545,7 +1535,7 @@ fn intra_frame_rdo_mode_decision<T: Pixel>(
     );
   });
 
-  if fi.config.speed_settings.fine_directional_intra
+  if fi.config.speed_settings.prediction.fine_directional_intra
     && bsize >= BlockSize::BLOCK_8X8
   {
     // Find the best angle delta for the current best prediction mode
@@ -1596,13 +1586,19 @@ fn intra_frame_rdo_mode_decision<T: Pixel>(
   best
 }
 
+/// # Panics
+///
+/// - If the block size is invalid for subsampling.
+#[profiling::function]
 pub fn rdo_cfl_alpha<T: Pixel>(
   ts: &mut TileStateMut<'_, T>, tile_bo: TileBlockOffset, bsize: BlockSize,
   luma_tx_size: TxSize, fi: &FrameInvariants<T>,
 ) -> Option<CFLParams> {
   let PlaneConfig { xdec, ydec, .. } = ts.input.planes[1].cfg;
   let uv_tx_size = bsize.largest_chroma_tx_size(xdec, ydec);
-  debug_assert!(bsize.subsampled_size(xdec, ydec) == uv_tx_size.block_size());
+  debug_assert!(
+    bsize.subsampled_size(xdec, ydec).unwrap() == uv_tx_size.block_size()
+  );
 
   let frame_bo = ts.to_frame_block_offset(tile_bo);
   let (visible_tx_w, visible_tx_h) = clip_visible_bsize(
@@ -1616,8 +1612,8 @@ pub fn rdo_cfl_alpha<T: Pixel>(
   if visible_tx_w == 0 || visible_tx_h == 0 {
     return None;
   };
-  let mut ac: Aligned<[i16; 32 * 32]> = Aligned::uninitialized();
-  luma_ac(&mut ac.data, ts, tile_bo, bsize, luma_tx_size, fi);
+  let mut ac = Aligned::<[MaybeUninit<i16>; 32 * 32]>::uninit_array();
+  let ac = luma_ac(&mut ac.data, ts, tile_bo, bsize, luma_tx_size, fi);
   let best_alpha: ArrayVec<i16, 2> = (1..3)
     .map(|p| {
       let &PlaneConfig { xdec, ydec, .. } = ts.rec.planes[p].plane_cfg;
@@ -1625,7 +1621,9 @@ pub fn rdo_cfl_alpha<T: Pixel>(
       let rec = &mut ts.rec.planes[p];
       let input = &ts.input_tile.planes[p];
       let po = tile_bo.plane_offset(rec.plane_cfg);
+      let mut edge_buf = Aligned::uninit_array();
       let edge_buf = get_intra_edges(
+        &mut edge_buf,
         &rec.as_const(),
         tile_bo,
         0,
@@ -1646,7 +1644,7 @@ pub fn rdo_cfl_alpha<T: Pixel>(
           &mut rec_region,
           uv_tx_size,
           fi.sequence.bit_depth,
-          &ac.data,
+          ac,
           IntraParam::Alpha(alpha),
           None,
           &edge_buf,
@@ -1691,17 +1689,24 @@ pub fn rdo_cfl_alpha<T: Pixel>(
 }
 
 /// RDO-based transform type decision
-/// If cw_checkpoint is None, a checkpoint for cw's (ContextWriter) current
+/// If `cw_checkpoint` is `None`, a checkpoint for cw's (`ContextWriter`) current
 /// state is created and stored for later use.
+///
+/// # Panics
+///
+/// - If a writer checkpoint is never created before or within the function.
+///   This should never happen and indicates a development error.
+/// - If the best RD found is negative.
+///   This should never happen and indicates a development error.
 pub fn rdo_tx_type_decision<T: Pixel>(
   fi: &FrameInvariants<T>, ts: &mut TileStateMut<'_, T>,
   cw: &mut ContextWriter, cw_checkpoint: &mut Option<ContextWriterCheckpoint>,
   mode: PredictionMode, ref_frames: [RefType; 2], mvs: [MotionVector; 2],
   bsize: BlockSize, tile_bo: TileBlockOffset, tx_size: TxSize, tx_set: TxSet,
-  tx_types: &[TxType],
+  tx_types: &[TxType], cur_best_rd: f64,
 ) -> (TxType, f64) {
   let mut best_type = TxType::DCT_DCT;
-  let mut best_rd = std::f64::MAX;
+  let mut best_rd = f64::MAX;
 
   let PlaneConfig { xdec, ydec, .. } = ts.input.planes[1].cfg;
   let is_chroma_block =
@@ -1723,6 +1728,7 @@ pub fn rdo_tx_type_decision<T: Pixel>(
   };
   let need_recon_pixel = tx_size.block_size() != bsize && !is_inter;
 
+  let mut first_iteration = true;
   for &tx_type in tx_types {
     // Skip unsupported transform types
     if av1_tx_used[tx_set as usize][tx_type as usize] == 0 {
@@ -1790,13 +1796,25 @@ pub fn rdo_tx_type_decision<T: Pixel>(
     } else {
       compute_distortion(fi, ts, bsize, is_chroma_block, tile_bo, true)
     };
+    cw.rollback(cw_checkpoint.as_ref().unwrap());
+
     let rd = compute_rd_cost(fi, rate, distortion);
+
+    if first_iteration {
+      // We use an optimization to early exit after testing the first
+      // transform type if the cost is higher than the existing best.
+      // The idea is that if this transform size is not better than he
+      // previous size, it is not worth testing remaining modes for this size.
+      if rd > cur_best_rd {
+        break;
+      }
+      first_iteration = false;
+    }
+
     if rd < best_rd {
       best_rd = rd;
       best_type = tx_type;
     }
-
-    cw.rollback(cw_checkpoint.as_ref().unwrap());
   }
 
   assert!(best_rd >= 0_f64);
@@ -1853,9 +1871,7 @@ fn rdo_partition_simple<T: Pixel, W: Writer>(
   child_modes: &mut ArrayVec<PartitionParameters, 4>,
 ) -> Option<f64> {
   debug_assert!(tile_bo.0.x < ts.mi_width && tile_bo.0.y < ts.mi_height);
-  let subsize = bsize.subsize(partition);
-
-  debug_assert!(subsize != BlockSize::BLOCK_INVALID);
+  let subsize = bsize.subsize(partition).unwrap();
 
   let cost = if bsize >= BlockSize::BLOCK_8X8 {
     let w: &mut W = if cw.bc.cdef_coded { w_post_cdef } else { w_pre_cdef };
@@ -1870,17 +1886,11 @@ fn rdo_partition_simple<T: Pixel, W: Writer>(
   let hbsh = subsize.height_mi(); // Half the block size height in blocks
   let four_partitions = [
     tile_bo,
+    TileBlockOffset(BlockOffset { x: tile_bo.0.x + hbsw, y: tile_bo.0.y }),
+    TileBlockOffset(BlockOffset { x: tile_bo.0.x, y: tile_bo.0.y + hbsh }),
     TileBlockOffset(BlockOffset {
-      x: tile_bo.0.x + hbsw as usize,
-      y: tile_bo.0.y,
-    }),
-    TileBlockOffset(BlockOffset {
-      x: tile_bo.0.x,
-      y: tile_bo.0.y + hbsh as usize,
-    }),
-    TileBlockOffset(BlockOffset {
-      x: tile_bo.0.x + hbsw as usize,
-      y: tile_bo.0.y + hbsh as usize,
+      x: tile_bo.0.x + hbsw,
+      y: tile_bo.0.y + hbsh,
     }),
   ];
 
@@ -1917,11 +1927,11 @@ fn rdo_partition_simple<T: Pixel, W: Writer>(
         offset,
         &mode_decision,
         rdo_type,
-        false,
+        None,
       );
       child_modes.push(mode_decision);
     } else {
-      //rd_cost_sum += std::f64::MAX;
+      //rd_cost_sum += f64::MAX;
       return None;
     }
   }
@@ -1929,7 +1939,13 @@ fn rdo_partition_simple<T: Pixel, W: Writer>(
   Some(cost + rd_cost_sum)
 }
 
-// RDO-based single level partitioning decision
+/// RDO-based single level partitioning decision
+///
+/// # Panics
+///
+/// - If the best RD found is negative.
+///   This should never happen, and indicates a development error.
+#[profiling::function]
 pub fn rdo_partition_decision<T: Pixel, W: Writer>(
   fi: &FrameInvariants<T>, ts: &mut TileStateMut<'_, T>,
   cw: &mut ContextWriter, w_pre_cdef: &mut W, w_post_cdef: &mut W,
@@ -1990,7 +2006,7 @@ pub fn rdo_partition_decision<T: Pixel, W: Writer>(
       if rd < best_rd {
         best_rd = rd;
         best_partition = partition;
-        best_pred_modes = child_modes.clone();
+        best_pred_modes.clone_from(&child_modes);
       }
     }
     cw.rollback(&cw_checkpoint);
@@ -2007,6 +2023,7 @@ pub fn rdo_partition_decision<T: Pixel, W: Writer>(
   }
 }
 
+#[profiling::function]
 fn rdo_loop_plane_error<T: Pixel>(
   base_sbo: TileSuperBlockOffset, offset_sbo: TileSuperBlockOffset,
   sb_w: usize, sb_h: usize, fi: &FrameInvariants<T>, ts: &TileStateMut<'_, T>,
@@ -2049,7 +2066,14 @@ fn rdo_loop_plane_error<T: Pixel>(
           // For loop filters, We intentionally use cdef_dist even with
           // `--tune Psnr`. Using SSE instead gives no PSNR gain but has a
           // significant negative impact on other metrics and visual quality.
-          cdef_dist_wxh_8x8(&src_region, &test_region, fi.sequence.bit_depth)
+          RawDistortion(cdef_dist_kernel(
+            &src_region,
+            &test_region,
+            8,
+            8,
+            fi.sequence.bit_depth,
+            fi.cpu_feature_level,
+          ) as u64)
             * bias
         } else {
           sse_wxh(
@@ -2068,10 +2092,15 @@ fn rdo_loop_plane_error<T: Pixel>(
   err * fi.dist_scale[pli]
 }
 
-// Passed in a superblock offset representing the upper left corner of
-// the LRU area we're optimizing.  This area covers the largest LRU in
-// any of the present planes, but may consist of a number of
-// superblocks and full, smaller LRUs in the other planes
+/// Passed in a superblock offset representing the upper left corner of
+/// the LRU area we're optimizing.  This area covers the largest LRU in
+/// any of the present planes, but may consist of a number of
+/// superblocks and full, smaller LRUs in the other planes
+///
+/// # Panics
+///
+/// - If both CDEF and LRF are disabled.
+#[profiling::function]
 pub fn rdo_loop_decision<T: Pixel, W: Writer>(
   base_sbo: TileSuperBlockOffset, fi: &FrameInvariants<T>,
   ts: &mut TileStateMut<'_, T>, cw: &mut ContextWriter, w: &mut W,
@@ -2143,6 +2172,102 @@ pub fn rdo_loop_decision<T: Pixel, W: Writer>(
   // improve at the reconstruction stage.
   let mut best_lrf_cost = [[-1.0; MAX_PLANES]; MAX_LRU_SIZE * MAX_LRU_SIZE];
 
+  // sub-setted region of the TileBlocks for our working frame area.
+  // Note that the size of this subset is what signals CDEF as to the
+  // actual coded size.
+  let mut tileblocks_subset = cw.bc.blocks.subregion_mut(
+    base_sbo.block_offset(0, 0).0.x,
+    base_sbo.block_offset(0, 0).0.y,
+    sb_w << SUPERBLOCK_TO_BLOCK_SHIFT,
+    sb_h << SUPERBLOCK_TO_BLOCK_SHIFT,
+  );
+
+  // cdef doesn't run on superblocks that are completely skipped.
+  // Determine which super blocks are marked as skipped so we can avoid running
+  // them. If all blocks are skipped, we can avoid some of the overhead related
+  // to setting up for cdef.
+  let mut cdef_skip = [true; MAX_SB_SIZE * MAX_SB_SIZE];
+  let mut cdef_skip_all = true;
+  if fi.sequence.enable_cdef {
+    for sby in 0..sb_h {
+      for sbx in 0..sb_w {
+        let blocks = tileblocks_subset.subregion(16 * sbx, 16 * sby, 16, 16);
+        let mut skip = true;
+        for y in 0..blocks.rows() {
+          for block in blocks[y].iter() {
+            skip &= block.skip;
+          }
+        }
+        cdef_skip[sby * MAX_SB_SIZE + sbx] = skip;
+        cdef_skip_all &= skip;
+      }
+    }
+  }
+
+  // Unlike cdef, loop restoration will run regardless of whether blocks are
+  // skipped or not. At the same time, the most significant improvement will
+  // generally be from un-skipped blocks, so lru is only performed if there are
+  // un-skipped blocks.
+  // This should be the same as `cdef_skip_all`, except when cdef is disabled.
+  let mut lru_skip_all = true;
+  let mut lru_skip = [[true; MAX_PLANES]; MAX_LRU_SIZE * MAX_LRU_SIZE];
+  if fi.sequence.enable_restoration {
+    if fi.config.speed_settings.lru_on_skip {
+      lru_skip_all = false;
+      lru_skip = [[false; MAX_PLANES]; MAX_LRU_SIZE * MAX_LRU_SIZE];
+    } else {
+      for pli in 0..planes {
+        // width, in sb, of an LRU in this plane
+        let lru_sb_w = 1 << ts.restoration.planes[pli].rp_cfg.sb_h_shift;
+        // height, in sb, of an LRU in this plane
+        let lru_sb_h = 1 << ts.restoration.planes[pli].rp_cfg.sb_v_shift;
+        for lru_y in 0..lru_h[pli] {
+          // number of LRUs vertically
+          for lru_x in 0..lru_w[pli] {
+            // number of LRUs horizontally
+
+            let loop_sbo = TileSuperBlockOffset(SuperBlockOffset {
+              x: lru_x * lru_sb_w,
+              y: lru_y * lru_sb_h,
+            });
+
+            if !ts.restoration.has_restoration_unit(
+              base_sbo + loop_sbo,
+              pli,
+              false,
+            ) {
+              continue;
+            }
+
+            let start = loop_sbo.block_offset(0, 0).0;
+            let size = TileSuperBlockOffset(SuperBlockOffset {
+              x: lru_sb_w,
+              y: lru_sb_h,
+            })
+            .block_offset(0, 0)
+            .0;
+
+            let blocks =
+              tileblocks_subset.subregion(start.x, start.y, size.x, size.y);
+            let mut skip = true;
+            for y in 0..blocks.rows() {
+              for block in blocks[y].iter() {
+                skip &= block.skip;
+              }
+            }
+            lru_skip[lru_y * MAX_LRU_SIZE + lru_x][pli] = skip;
+            lru_skip_all &= skip;
+          }
+        }
+      }
+    }
+  }
+
+  // Return early if all blocks are skipped for lru and cdef.
+  if lru_skip_all && cdef_skip_all {
+    return;
+  }
+
   // Loop filter RDO is an iterative process and we need temporary
   // scratch data to hold the results of deblocking, cdef, and the
   // loop reconstruction filter so that each can be partially updated
@@ -2158,16 +2283,6 @@ pub fn rdo_loop_decision<T: Pixel, W: Writer>(
     })
     .scratch_copy();
 
-  // sub-setted region of the TileBlocks for our working frame area.
-  // Note that the size of this subset is what signals CDEF as to the
-  // actual coded size.
-  let mut tileblocks_subset = cw.bc.blocks.subregion_mut(
-    base_sbo.block_offset(0, 0).0.x,
-    base_sbo.block_offset(0, 0).0.y,
-    sb_w << SUPERBLOCK_TO_BLOCK_SHIFT,
-    sb_h << SUPERBLOCK_TO_BLOCK_SHIFT,
-  );
-
   // const, no need to copy, just need the subregion (but do zero the
   // origin to match the other copies/new backing frames).
   let src_subset = ts
@@ -2212,8 +2327,8 @@ pub fn rdo_loop_decision<T: Pixel, W: Writer>(
   }
 
   let mut cdef_work =
-    if fi.sequence.enable_cdef { Some(rec_subset.clone()) } else { None };
-  let mut lrf_work = if fi.sequence.enable_restoration {
+    if !cdef_skip_all { Some(rec_subset.clone()) } else { None };
+  let mut lrf_work = if !lru_skip_all {
     Some(Frame {
       planes: {
         let new_plane = |pli: usize| {
@@ -2261,6 +2376,11 @@ pub fn rdo_loop_decision<T: Pixel, W: Writer>(
     {
       for sby in 0..sb_h {
         for sbx in 0..sb_w {
+          // determine whether this superblock can be skipped
+          if cdef_skip[sby * MAX_SB_SIZE + sbx] {
+            continue;
+          }
+
           let prev_best_index = best_index[sby * sb_w + sbx];
           let mut best_cost = -1.;
           let mut best_new_index = -1i8;
@@ -2310,7 +2430,7 @@ pub fn rdo_loop_decision<T: Pixel, W: Writer>(
               } {
                 // We have a valid LRU, apply LRF, compute error
                 match best_lrf[lru_y * lru_w[pli] + lru_x][pli] {
-                  RestorationFilter::None {} => {
+                  RestorationFilter::None => {
                     err += rdo_loop_plane_error(
                       base_sbo,
                       loop_sbo,
@@ -2456,7 +2576,7 @@ pub fn rdo_loop_decision<T: Pixel, W: Writer>(
     if let Some(lrf_ref) = &mut lrf_work.as_mut() {
       let lrf_input = if cdef_work.is_some() {
         // When CDEF is enabled, we pull from the CDEF output
-        &cdef_work.as_ref().unwrap()
+        cdef_work.as_ref().unwrap()
       } else {
         // When CDEF is disabled, we pull from the [optionally
         // deblocked] reconstruction
@@ -2474,6 +2594,12 @@ pub fn rdo_loop_decision<T: Pixel, W: Writer>(
           // number of LRUs vertically
           for lru_x in 0..lru_w[pli] {
             // number of LRUs horizontally
+
+            // determine whether this lru should be skipped
+            if lru_skip[lru_y * MAX_LRU_SIZE + lru_x][pli] {
+              continue;
+            }
+
             let loop_sbo = TileSuperBlockOffset(SuperBlockOffset {
               x: lru_x * lru_sb_w,
               y: lru_y * lru_sb_h,
