@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2021, The rav1e contributors. All rights reserved
+// Copyright (c) 2018-2022, The rav1e contributors. All rights reserved
 //
 // This source code is subject to the terms of the BSD 2 Clause License and
 // the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -11,9 +11,14 @@
 #![cfg_attr(fuzzing, allow(unused))]
 
 use crate::color::ChromaSampling;
+
+use crate::api::config::GrainTableSegment;
 use crate::util::Pixel;
 use crate::*;
+
+use arrayvec::ArrayVec;
 use interpolate_name::interpolate_test;
+use log::debug;
 use rand::{Rng, SeedableRng};
 use rand_chacha::ChaChaRng;
 use std::collections::VecDeque;
@@ -33,7 +38,7 @@ fn fill_frame<T: Pixel>(ra: &mut ChaChaRng, frame: &mut Frame<T>) {
     let stride = plane.cfg.stride;
     for row in plane.data.chunks_mut(stride) {
       for pixel in row {
-        let v: u8 = ra.gen();
+        let v: u8 = ra.random();
         *pixel = T::cast_from(v);
       }
     }
@@ -64,11 +69,12 @@ pub(crate) trait TestDecoder<T: Pixel> {
   where
     Self: Sized;
   fn encode_decode(
-    &mut self, w: usize, h: usize, speed: usize, quantizer: usize,
+    &mut self, verify: bool, w: usize, h: usize, speed: u8, quantizer: usize,
     limit: usize, bit_depth: usize, chroma_sampling: ChromaSampling,
     min_keyint: u64, max_keyint: u64, switch_frame_interval: u64,
     low_latency: bool, error_resilient: bool, bitrate: i32,
     tile_cols_log2: usize, tile_rows_log2: usize, still_picture: bool,
+    grain_table: Option<Vec<GrainTableSegment>>,
   ) {
     let mut ra = ChaChaRng::from_seed([0; 32]);
 
@@ -88,6 +94,7 @@ pub(crate) trait TestDecoder<T: Pixel> {
       tile_cols_log2,
       tile_rows_log2,
       still_picture,
+      grain_table,
     );
 
     debug!(
@@ -128,6 +135,7 @@ pub(crate) trait TestDecoder<T: Pixel> {
             h,
             chroma_sampling,
             bit_depth,
+            verify,
           ) {
             DecodeResult::Done => {
               break;
@@ -146,7 +154,7 @@ pub(crate) trait TestDecoder<T: Pixel> {
   }
   fn decode_packet(
     &mut self, packet: &[u8], rec_fifo: &mut VecDeque<Frame<T>>, w: usize,
-    h: usize, chroma_sampling: ChromaSampling, bit_depth: usize,
+    h: usize, chroma_sampling: ChromaSampling, bit_depth: usize, verify: bool,
   ) -> DecodeResult;
 }
 
@@ -168,11 +176,11 @@ pub fn compare_plane<T: Ord + std::fmt::Debug>(
 }
 
 fn setup_encoder<T: Pixel>(
-  w: usize, h: usize, speed: usize, quantizer: usize, bit_depth: usize,
+  w: usize, h: usize, speed: u8, quantizer: usize, bit_depth: usize,
   chroma_sampling: ChromaSampling, min_keyint: u64, max_keyint: u64,
   switch_frame_interval: u64, low_latency: bool, error_resilient: bool,
   bitrate: i32, tile_cols_log2: usize, tile_rows_log2: usize,
-  still_picture: bool,
+  still_picture: bool, grain_table: Option<Vec<GrainTableSegment>>,
 ) -> Context<T> {
   assert!(bit_depth == 8 || std::mem::size_of::<T>() > 1);
   let mut enc = EncoderConfig::with_speed_preset(speed);
@@ -190,6 +198,7 @@ fn setup_encoder<T: Pixel>(
   enc.tile_cols = 1 << tile_cols_log2;
   enc.tile_rows = 1 << tile_rows_log2;
   enc.still_picture = still_picture;
+  enc.film_grain_params = grain_table;
 
   let threads = if cfg!(fuzzing) { 1 } else { 2 };
 
@@ -202,7 +211,7 @@ fn setup_encoder<T: Pixel>(
 static DIMENSION_OFFSETS: &[(usize, usize)] =
   &[(0, 0), (4, 4), (8, 8), (16, 16)];
 
-fn speed(s: usize, decoder: &str) {
+fn speed(s: u8, decoder: &str) {
   let quantizer = 100;
   let limit = 5;
   let w = 64;
@@ -213,6 +222,7 @@ fn speed(s: usize, decoder: &str) {
     let h = h + b.1;
     let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
     dec.encode_decode(
+      true,
       w,
       h,
       s,
@@ -229,6 +239,7 @@ fn speed(s: usize, decoder: &str) {
       0,
       0,
       false,
+      None,
     );
   }
 }
@@ -305,6 +316,7 @@ fn dimension(w: usize, h: usize, decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -321,6 +333,7 @@ fn dimension(w: usize, h: usize, decoder: &str) {
     0,
     0,
     still_picture,
+    None,
   );
 }
 
@@ -333,6 +346,7 @@ fn quantizer(decoder: &str, q: usize) {
   for b in DIMENSION_OFFSETS.iter() {
     let mut dec = get_decoder::<u8>(decoder, b.0, b.1);
     dec.encode_decode(
+      true,
       w + b.0,
       h + b.1,
       speed,
@@ -349,6 +363,7 @@ fn quantizer(decoder: &str, q: usize) {
       0,
       0,
       false,
+      None,
     );
   }
 }
@@ -382,6 +397,7 @@ fn bitrate(decoder: &str) {
     for &r in [100, 1000, 10_000].iter() {
       let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
       dec.encode_decode(
+        true,
         w,
         h,
         speed,
@@ -398,6 +414,7 @@ fn bitrate(decoder: &str) {
         0,
         0,
         false,
+        None,
       );
     }
   }
@@ -414,6 +431,7 @@ fn keyframes(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -430,6 +448,7 @@ fn keyframes(decoder: &str) {
     0,
     0,
     false,
+    None,
   );
 }
 
@@ -445,6 +464,7 @@ fn reordering(decoder: &str) {
   for keyint in &[4, 5, 6] {
     let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
     dec.encode_decode(
+      true,
       w,
       h,
       speed,
@@ -461,6 +481,7 @@ fn reordering(decoder: &str) {
       0,
       0,
       false,
+      None,
     );
   }
 }
@@ -478,6 +499,7 @@ fn reordering_short_video(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -494,6 +516,7 @@ fn reordering_short_video(decoder: &str) {
     0,
     0,
     false,
+    None,
   );
 }
 
@@ -509,6 +532,7 @@ fn error_resilient(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -525,6 +549,7 @@ fn error_resilient(decoder: &str) {
     0,
     0,
     false,
+    None,
   );
 }
 
@@ -540,6 +565,7 @@ fn error_resilient_reordering(decoder: &str) {
   for keyint in &[4, 5, 6] {
     let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
     dec.encode_decode(
+      true,
       w,
       h,
       speed,
@@ -556,6 +582,7 @@ fn error_resilient_reordering(decoder: &str) {
       0,
       0,
       false,
+      None,
     );
   }
 }
@@ -572,6 +599,7 @@ fn switch_frame(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -588,6 +616,7 @@ fn switch_frame(decoder: &str) {
     0,
     0,
     false,
+    None,
   );
 }
 
@@ -603,6 +632,7 @@ fn odd_size_frame_with_full_rdo(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -619,6 +649,7 @@ fn odd_size_frame_with_full_rdo(decoder: &str) {
     0,
     0,
     false,
+    None,
   );
 }
 
@@ -635,6 +666,7 @@ fn low_bit_depth(decoder: &str) {
   // 8-bit
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -651,6 +683,7 @@ fn low_bit_depth(decoder: &str) {
     0,
     0,
     false,
+    None,
   );
 }
 
@@ -663,6 +696,7 @@ fn high_bit_depth(decoder: &str, depth: usize) {
 
   let mut dec = get_decoder::<u16>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -679,6 +713,7 @@ fn high_bit_depth(decoder: &str, depth: usize) {
     0,
     0,
     false,
+    None,
   );
 }
 
@@ -708,8 +743,8 @@ fn chroma_sampling(decoder: &str, cs: ChromaSampling) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
-    w, h, speed, quantizer, limit, 8, cs, 15, 15, 0, true, false, 0, 0, 0,
-    false,
+    true, w, h, speed, quantizer, limit, 8, cs, 15, 15, 0, true, false, 0, 0,
+    0, false, None,
   );
 }
 
@@ -744,6 +779,7 @@ fn tile_encoding_with_stretched_restoration_units(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -760,6 +796,7 @@ fn tile_encoding_with_stretched_restoration_units(decoder: &str) {
     2,
     2,
     false,
+    None,
   );
 }
 
@@ -774,6 +811,7 @@ fn still_picture_mode(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -790,6 +828,7 @@ fn still_picture_mode(decoder: &str) {
     0,
     0,
     true,
+    None,
   );
 }
 
@@ -817,6 +856,7 @@ fn rdo_loop_decision_lrf_sanity(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -833,6 +873,7 @@ fn rdo_loop_decision_lrf_sanity(decoder: &str) {
     0,
     0,
     false,
+    None,
   );
 }
 
@@ -848,6 +889,7 @@ fn rdo_loop_decision_cdef_sanity(decoder: &str) {
 
   let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
   dec.encode_decode(
+    true,
     w,
     h,
     speed,
@@ -864,5 +906,168 @@ fn rdo_loop_decision_cdef_sanity(decoder: &str) {
     0,
     0,
     false,
+    None,
+  );
+}
+
+#[cfg_attr(feature = "decode_test", interpolate_test(aom, "aom"))]
+#[cfg_attr(feature = "decode_test_dav1d", interpolate_test(dav1d, "dav1d"))]
+#[ignore]
+fn film_grain_table_luma_only(decoder: &str) {
+  let quantizer = 100;
+  let limit = 5; // Include inter frames
+  let speed = 10;
+  let w = 64;
+  let h = 80;
+
+  let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
+  dec.encode_decode(
+    false,
+    w,
+    h,
+    speed,
+    quantizer,
+    limit,
+    8,
+    Default::default(),
+    15,
+    15,
+    0,
+    true,
+    false,
+    0,
+    0,
+    0,
+    false,
+    Some(vec![GrainTableSegment {
+      start_time: 0,
+      end_time: 9223372036854775807,
+      scaling_points_y: ArrayVec::from([
+        [0, 20],
+        [20, 5],
+        [39, 4],
+        [59, 3],
+        [78, 3],
+        [98, 3],
+        [118, 3],
+        [137, 3],
+        [157, 3],
+        [177, 3],
+        [196, 3],
+        [216, 4],
+        [235, 4],
+        [255, 4],
+      ]),
+      scaling_points_cb: ArrayVec::new(),
+      scaling_points_cr: ArrayVec::new(),
+      scaling_shift: 8,
+      ar_coeff_lag: 0,
+      ar_coeffs_y: ArrayVec::new(),
+      ar_coeffs_cb: ArrayVec::try_from([0].as_slice()).unwrap(),
+      ar_coeffs_cr: ArrayVec::try_from([0].as_slice()).unwrap(),
+      ar_coeff_shift: 6,
+      cb_mult: 0,
+      cb_luma_mult: 0,
+      cb_offset: 0,
+      cr_mult: 0,
+      cr_luma_mult: 0,
+      cr_offset: 0,
+      overlap_flag: true,
+      chroma_scaling_from_luma: false,
+      grain_scale_shift: 0,
+      random_seed: 7391,
+    }]),
+  );
+}
+
+#[cfg_attr(feature = "decode_test", interpolate_test(aom, "aom"))]
+#[cfg_attr(feature = "decode_test_dav1d", interpolate_test(dav1d, "dav1d"))]
+#[ignore]
+fn film_grain_table_chroma(decoder: &str) {
+  let quantizer = 100;
+  let limit = 5; // Include inter frames
+  let speed = 10;
+  let w = 64;
+  let h = 80;
+
+  let mut dec = get_decoder::<u8>(decoder, w as usize, h as usize);
+  dec.encode_decode(
+    false,
+    w,
+    h,
+    speed,
+    quantizer,
+    limit,
+    8,
+    Default::default(),
+    15,
+    15,
+    0,
+    true,
+    false,
+    0,
+    0,
+    0,
+    false,
+    Some(vec![GrainTableSegment {
+      start_time: 0,
+      end_time: 9223372036854775807,
+      scaling_points_y: ArrayVec::from([
+        [0, 0],
+        [20, 4],
+        [39, 3],
+        [59, 3],
+        [78, 3],
+        [98, 3],
+        [118, 4],
+        [137, 4],
+        [157, 4],
+        [177, 4],
+        [196, 4],
+        [216, 5],
+        [235, 5],
+        [255, 5],
+      ]),
+      scaling_points_cb: ArrayVec::from([
+        [0, 0],
+        [28, 0],
+        [57, 0],
+        [85, 0],
+        [113, 0],
+        [142, 0],
+        [170, 0],
+        [198, 0],
+        [227, 0],
+        [255, 1],
+      ]),
+      scaling_points_cr: ArrayVec::from([
+        [0, 0],
+        [28, 0],
+        [57, 0],
+        [85, 0],
+        [113, 0],
+        [142, 0],
+        [170, 0],
+        [198, 0],
+        [227, 0],
+        [255, 1],
+      ]),
+      scaling_shift: 8,
+      ar_coeff_lag: 0,
+      ar_coeffs_y: ArrayVec::new(),
+      ar_coeffs_cb: ArrayVec::try_from([0].as_slice()).unwrap(),
+      ar_coeffs_cr: ArrayVec::try_from([0].as_slice()).unwrap(),
+      ar_coeff_shift: 6,
+      cb_mult: 128,
+      cb_luma_mult: 192,
+      cb_offset: 256,
+      cr_mult: 128,
+      cr_luma_mult: 192,
+      cr_offset: 256,
+      overlap_flag: true,
+      chroma_scaling_from_luma: false,
+      grain_scale_shift: 0,
+      random_seed: 7391,
+    }]),
   );
 }
